package edu.uoc.tcfj2ee_gatd.configuracio;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

import edu.uoc.tcfj2ee_gatd.model.comu.Util;

/**
 * Dades de configuració del sistema.
 * Testimoni de l'instal·lació del programari i càrrega de la
 * seva configuració inicial
 * @author Jordi Tejero
 *
 */
@Entity
public class Configuracio implements Serializable {

	/**
	 * Versió de serialització. 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Valor únic de la clau primària
	 */
	static final int ID = 1;

	/**
	 * Clau primària. Sempre és 1. 
	 */
	@Id
	private int id;	
	
	/**
	 * Data de la instal·lació
	 */
	@Temporal(TemporalType.TIMESTAMP)
	private Date dataInst;
	
	/**
	 * Nombre d'hores a partir de les que una petició no acceptada dona
	 * alerta
	 */
	private float llindarAlertaAtencio;
	
	/**
	 * Nombre d'hores a partir de les que una petició pendent dona alerta
	 */
	private float llindarAlertaFi;
	
	/**
	 * L'adreça del servidor. S'inclou als missatges que s'envien als tècnics.
	 */
	@Column(nullable = false)	
	private String servidor;
	
	/**
	 * El port del servidor. S'inclou als missatges que s'envien als tècnics.
	 * Pot ser <code>null</code>
	 */
	@Column(length = 6)
	private String port;
	
	/**
	 * Servidor de correu electrònic que es fa servir per a enviar els missatges
	 */
	@Column(nullable = false)
	private String mailHost;
	
	/**
	 * Nom d'usuari al servidor de correu electrònic.
	 * Por ser <code>null</code> si no call validar-se.
	 */
	private String mailUsuari;
	
	/**
	 * Clau d'accés al servidor de correu electrònic.
	 * Por ser <code>null</code> si no call validar-se.
	 */
	private String mailClau;

	/**
	 * Adreça del remitent dels missatges de correu electrònic que
	 * el sistema envia
	 */
	private String mailRemitent;
	
	
	/**
	 * Dona l'adreça del servidor
	 * @return el valor de l'atribut <code>servidor</code>
	 */
	public String getServidor() {
		return this.servidor;
	}



	/**
	 * Accessor d'escriptura a l'atribut servidor
	 * @param servidor el nou valor de l'atribut servidor
	 */
	public void setServidor(String servidor) {
		this.servidor = servidor;
	}



	/**
	 * Accessor de lectura de l'atribut port
	 * @return el valor de l'atribut port
	 */
	public String getPort() {
		return this.port;
	}



	/**
	 * Accessor d'escriptura a l'atribut port
	 * @param port el nou valor de l'atribut port
	 */
	public void setPort(String port) {
		this.port = port;
	}



	/**
	 * Accessor de lectura de l'atribut mailHost
	 * @return el valor de l'atribut <code>mailHost</code>
	 */
	public String getMailHost() {
		return this.mailHost;
	}



	/**
	 * Accessor d'escriptura a l'atribut mailHost
	 * @param mailHost el nou valor de l'atribut mailHost
	 */
	public void setMailHost(String mailHost) {
		this.mailHost = mailHost;
	}



	/**
	 * Accessor de lectura de l'atribut mailUsuari
	 * @return el valor de l'atribut <code>mailUsuari</code>
	 */
	public String getMailUsuari() {
		return this.mailUsuari;
	}



	/**
	 * Accessor d'escriptura a l'atribut mailUsuari
	 * @param mailUsuari el nou valor de l'atribut mailUsuari
	 */
	public void setMailUsuari(String mailUsuari) {
		this.mailUsuari = mailUsuari;
	}



	/**
	 * Accessor de lectura de l'atribut mailClau
	 * @return el valor de l'atribut <code>mailClau</code>
	 */
	public String getMailClau() {
		return this.mailClau;
	}



	/**
	 * Accessor d'escriptura a l'atribut mailClau
	 * @param mailClau el nou valor de l'atribut mailClau
	 */
	public void setMailClau(String mailClau) {
		this.mailClau = mailClau;
	}



	/**
	 * Dona l'adreça del remitent dels missatges e-mail
	 * @return l'adreça del remitent
	 */
	public String getMailRemitent() {
		return this.mailRemitent;
	}



	/**
	 * Estableix l'adreça del remitent dels missatges e-mail
	 * @param mailRemitent el nou valor de l'atribut mailRemitent
	 */
	public void setMailRemitent(String mailRemitent) {
		this.mailRemitent = mailRemitent;
	}



	/**
	 * Accessor de lectura de l'atribut dataInst
	 * @return el valor de l'atribut <code>dataInst</code>
	 */
	public Date getDataInst() {
		return this.dataInst;
	}


	
	/**
	 * Accessor de lectura de l'atribut id.
	 * @return el valor de l'atribut <code>id,</code> que sempre és 1.
	 */
	public int getId() {
		return this.id;
	}

	/**
	 * Accessor de lectura de l'atribut llindarAlertaFi
	 * @return el valor de l'atribut <code>llindarAlertaFi</code>
	 */
	public float getLlindarAlertaFi() {
		return this.llindarAlertaFi;
	}



	/**
	 * Accessor d'escriptura a l'atribut llindarAlertaFi
	 * @param llindarAlertaFi el nou valor de l'atribut llindarAlertaFi
	 */
	public void setLlindarAlertaFi(float llindarAlertaFi) {
		this.llindarAlertaFi = llindarAlertaFi;
	}



	/**
	 * Accessor de lectura de l'atribut llindarAlertaAtencio
	 * @return el valor de l'atribut <code>llindarAlertaAtencio</code>
	 */
	public float getLlindarAlertaAtencio() {
		return this.llindarAlertaAtencio;
	}



	/**
	 * Accessor d'escriptura a l'atribut llindarAlertaAtencio
	 * @param llindarAlertaAtencio el nou valor de l'atribut llindarAlertaAtencio
	 */
	public void setLlindarAlertaAtencio(float llindarAlertaAtencio) {
		this.llindarAlertaAtencio = llindarAlertaAtencio;
	}



	/**
	 * Constructor
	 */
	public Configuracio() {
		this.id = Configuracio.ID;
		this.dataInst = new Date();
	}

	/*  Representació textual de l'objecte, per a depuració i proves.
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return 
		"[" + this.getClass().getSimpleName() + "]" + Util.LS +
		"Servidor: " + this.servidor + Util.LS +
		"Llindar alertar atenció: " + this.llindarAlertaAtencio + Util.LS +
		"Llindar alerta finaltizació: " + this.llindarAlertaFi + Util.LS +
		"Servidor e-mail: " + this.mailHost + Util.LS +
		"Usuari e-mail: " + this.mailUsuari + Util.LS +
		"Clau e-mail: " + this.mailClau
		;
	}
	
	
}
