/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.configuracio;

import java.io.InputStream;
import java.text.ParseException;
import java.util.Date;
import java.util.Properties;

import javax.annotation.PostConstruct;
import javax.ejb.EJB;
import javax.ejb.Singleton;
import javax.ejb.Startup;

import edu.uoc.tcfj2ee_gatd.model.atencio.entitats.Tecnic;
import edu.uoc.tcfj2ee_gatd.model.comu.EJBGestor;
import edu.uoc.tcfj2ee_gatd.model.comu.Util;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Client;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Controlador;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Peticio;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Queixa;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Seu;
import edu.uoc.tcfj2ee_gatd.model.usuaris.GATDExcepcioUsuaris;
import edu.uoc.tcfj2ee_gatd.model.usuaris.GestorUsuaris;
import edu.uoc.tcfj2ee_gatd.model.usuaris.entitats.Adreça;
import edu.uoc.tcfj2ee_gatd.model.usuaris.entitats.Contacte;
import edu.uoc.tcfj2ee_gatd.model.usuaris.entitats.DistrictePostal;
import edu.uoc.tcfj2ee_gatd.model.usuaris.entitats.DocIdentitat;



/**
 * Carrega la configuració inicial de l'arxiu
 * config.properties. Crea unes quantes entitats predefinides.
 * <p>
 * Publica les dades de configuració.
 * </p>
 * @author Jordi Tejero
 *
 */
@Singleton
@Startup
public class GestorConfiguracio extends EJBGestor {
	
	/**
	 * Les dades de configuració del sistema
	 */
	private static Configuracio configuracio;
	
	/**
	 * EJB de negoci del subsistema d'usuaris
	 */
	@EJB
	private GestorUsuaris gUsuaris;		
		
	
	/**
	 * Si el programa encara no ha estat instal·lat, carrega la
	 * configuració inicial de l'arxiu de propietats
	 * @throws GATDExcepcioConfig cas d'error llegint de l'arxiu de propietats 
	 */
	@PostConstruct
	public void installat() throws GATDExcepcioConfig {
		
		Configuracio config = this.getEM().find(
				edu.uoc.tcfj2ee_gatd.configuracio.Configuracio.class, 
				Configuracio.ID);
		if(config == null) {
			this.getLogger().info(
				"--- GATD. Carregar configuració inicial");
			config = this.carrega();
			this.getLogger().info(
				"Dades de configuració del sistema " + config);
			this.getEM().persist(config);
		}
		else
			this.getLogger().info(
				"--- GATD. Configuració inicial ja carregada el " + 
				config.getDataInst());
		
		GestorConfiguracio.configuracio = config;
	}
	
	
	
	/**
	 * Dona una referència a la configuració del sistema. 
	 * 
	 * @return la configuració
	 */
	public static Configuracio getConfig() {
		return GestorConfiguracio.configuracio;
	}



	/**
	 * Càrrega de la configuració inicial.
	 * El mètode és cridat per contenidor després 
	 * inicialitzar l'objecte
	 * 
	 * @return la configuració del sistema llegida de l'arxiu de
	 *  propietats
	 * 
	 * @throws GATDExcepcioConfig cas de no poder llegir les propietats
	 */
	private Configuracio carrega() throws GATDExcepcioConfig  {
		
		Configuracio config;
		
		Properties prp = new Properties();
		
		InputStream in =
			this.getClass().getClassLoader()
			.getResourceAsStream(Constants.CONF_PROPS);
		if(in == null)
			throw new GATDExcepcioConfig(
					"** L'arxiu de propietats " + Constants.CONF_PROPS 
					+ " no és al classpath");
		
		try {			
			prp.load(in);
			in.close();

			config = this.carregaConfig(prp);
	        // this.carregaIdiomes(prp);
	        this.carregaSeus(prp);
	        this.carregaDP(prp);
	        this.carregaClients(prp);       
	        this.carregaControladors(prp);
	        this.carregaTecnics(prp);
	        this.carregaPeticions(prp);			
			
		}
		catch(GATDExcepcioConfig e) {
		    throw e;
		}
		catch (Throwable t) {
			throw new GATDExcepcioConfig(t, "err_excp_Inesperat", t.toString());			
		}
		
		return config;
	}
	
	/**
	 * Carrega les dades de configuració del sistema, a partir d'un
	 * subconjunt de propietats "config.*"
	 * @param prp les propietats on trobem el subconjunt
	 * @return un nou objecte <code>Configuracio</code> amb les propietats 
	 *  establertes segons les "config.*" corresponents
	 */
	private Configuracio carregaConfig(Properties prp) {
		Configuracio config = new Configuracio();
		final AjudantPrp aj = new AjudantPrp(prp, "config");
		config.setServidor(aj.get("servidor"));
		config.setPort(aj.get("port"));
		config.setLlindarAlertaAtencio(aj.getFloat("alerta.atencio.llindar"));
		config.setLlindarAlertaFi(aj.getFloat("alerta.fi.llindar"));
		config.setMailHost(aj.get("mail.host"));
		config.setMailUsuari(aj.get("mail.usuari"));
		config.setMailClau(aj.get("mail.clau"));
		config.setMailRemitent(aj.get("mail.remitent"));
		
		return config;
	}
	
	/**
	 * Carregar el idiomes predefinits
	 * @param prp el conjunt de propietats amb la configuració
	 */
	@SuppressWarnings("unused")
	/*
	private void carregaIdiomes(Properties prp) {
		this.getLogger().info("Carregar Idiomes");
		final AjudantPrp aj = new AjudantPrp(prp, "idiomes");
		for(int i = 1; i <= aj.getQuants(); ++i) {
			AjudantPrp ajF = aj.nouFill(i);
			Idioma idm = new Idioma(ajF.get("codi"), 
						ajF.get("nom"),
						ajF.getBol("omissio"));
			this.getLogger().finer("Carregat idioma: " + idm);			
			this.alta(idm);
		}
		this.getLogger().info("Carregat " + aj.getQuants() + " idiomes");
	}
	 */
	/**
	 * Carregar les seus predefinides
	 * @param prp el conjunt de propietats amb la configuració
	 * @throws GATDExcepcioConfig si alguna seu no està ben especificada
	 */
	private void carregaSeus(Properties prp) throws GATDExcepcioConfig {
		this.getLogger().info("Carregar Seus");
		final AjudantPrp aj = new AjudantPrp(prp, "seus");
		for(int i = 1; i <= aj.getQuants(); ++i) {
			AjudantPrp ajF = aj.nouFill(i);
			Seu s = new Seu(ajF.get("codi"), ajF.get("nom"), 
					this.carregaAdreça(ajF, null));
			this.getLogger().finer("Donar d'alta seu " + s);
			this.alta(s);
		}
		this.getLogger().info(
				"Carregat " + aj.getQuants() + " Seus");		
	}


	/**
	 * Crea una adreça amb les propietats "*.adr.*" d'un ajudant de
	 * propietats.
	 * <p>
	 * Si dites propietats no son a l'ajudant, retorna una adreça per
	 * omissió.
	 * </p>
	 * @param aj l'ajudant de propietats
	 * @param omissio adreça per omissió 
	 * @return <code>Adreça</code> creada, o <code>omissio</code>
	 *  si les propietats "*.adr.*" no existeixen o no indiquen
	 *  un codi postal
	 * @throws GATDExcepcioConfig si l'adreça no té codi postal
	 */
	private Adreça carregaAdreça(AjudantPrp aj, Adreça omissio) {
	    Adreça a;
		AjudantPrp ajF = aj.nouFill("adr");
		
        this.getLogger().finer("Muntar adreça " + ajF.getPrefix());		
		String cp = ajF.get("cp");
		if(cp == null)
		    a = omissio;
		else {
    		DistrictePostal dp = this.gUsuaris.donamDP(cp);
    		a = new Adreça(ajF.get("poblacio"), dp, ajF.get("carrer"), 
    				ajF.get("numero"), ajF.get("pis"), ajF.get("porta"));
		}
		this.getLogger().finer(
			"Muntat adreça " + ajF.getPrefix() + ": " + a);
		
		return a;
	}
	
	/**
	 * Carrega els districtes postals
	 * @param prp arxiu de propietats font
	 * @throws GATDExcepcioConfig si el districte no té codi
	 */
	private void carregaDP(Properties prp) {
		this.getLogger().info("Carregar Districtes Postals");
		final AjudantPrp aj = new AjudantPrp(prp, "dp");
		for(int i = 1; i <= aj.getQuants(); ++i) {
			AjudantPrp ajF = aj.nouFill(i);
			String cp = ajF.get("codi");
			this.getLogger().finer("Afegir DP " + cp);
			DistrictePostal dp = this.gUsuaris.donamDP(cp);
			dp.setSeu(this.getEM().find(Seu.class, ajF.get("seu")));
			this.alta(dp);
		}
		this.getLogger().info(
				"Carregat " + aj.getQuants() + " Districtes Postals");
	}
	
	/**
	 * Carrega els clients
	 * @param prp arxiu de propietats font
	 * @throws GATDExcepcioUsuaris si no es pot donar d'alta un Client
	 * @throws GATDExcepcioConfig cas de mancar quelcom
	 */
	private void carregaClients(Properties prp) throws GATDExcepcioUsuaris  {
		this.getLogger().info("------ Carregar Clients");
		final AjudantPrp aj = new AjudantPrp(prp, "clients");
		for(int i = 1; i <= aj.getQuants(); ++i) {
			AjudantPrp ajF = aj.nouFill(i);
			Client c = new Client(
				this.carregaUsuari(ajF), 
				this.carregaClau(ajF),
				this.carregaDI(ajF),
				this.carregaContacte(ajF, null),
				this.carregaAdreça(ajF, null),
				this.carregaIdioma(ajF)				
			);
			this.getLogger().finer("Donar d'alta client " + c);
			this.gUsuaris.alta(c);
			// this.alta(c);
		}
		this.getLogger().info(
				"Carregat " + aj.getQuants() + " clients");
	}	
	
	/**
	 * Carrega un codi d'usuari d'un grup de propietats
	 * @param aj ajudant per accedir al grup de propietats
	 * @return [aj].usuari
	 */
	private String carregaUsuari(AjudantPrp aj) {
		return aj.get("usuari");
	}

	/**
	 * Carrega una clau d'accés d'un grup de propietats
	 * @param aj ajudant per accedir al grup de propietats
	 * @return [aj].clau
	 */
	private String carregaClau(AjudantPrp aj) {
		return aj.get("clau");
	}
	
	
	/**
	 * Carrega un document d'identitat d'un grup de propietats
	 * @param aj ajudant per accedir al grup
	 * @return document d'identitat D amb D.tipus = [aj].di.tipus
	 *  i D.numero = [aj].di.numero
	 */
	private DocIdentitat carregaDI(AjudantPrp aj) {
		return new DocIdentitat(aj.get("di.tipus"), aj.get("di.numero"));
	}
	
	/**
	 * Carrega un codi de idioma d'un grup de propietat
	 * @param aj ajudant per accedir al grup de propietats
	 * @return cadena amb el codi del idioma
	 */
	private String carregaIdioma(AjudantPrp aj) {
		return aj.get("idioma");
	}
	
	/**
	 * Crea unes dades de contacte amb les propietats "*.cnt.*" d'un ajudant de
	 * propietats.
	 * <p>
	 * Si l'ajudant no conté les propietats indicades, retorna un objecte
	 * per omissió
	 * </p>
	 * @param aj l'ajudant de propietats
	 * @param omissio les dades de contacte per omissió. Poden ser
	 *  <code>null</code>
	 * @return <code>Contacte</code> creat, o <code>omissio</code>
	 *  si l'ajudant no conté la propietat "*.cnt.nom"
	 */
	private Contacte carregaContacte(AjudantPrp aj, Contacte omissio) {
	    Contacte c;
	    
		AjudantPrp ajF = aj.nouFill("cnt");
		this.getLogger().finer("Muntar Contacte " + ajF.getPrefix());
		String nom = ajF.get("nom");
		if(nom == null)
		    c = omissio;
		else
		    c = new Contacte(nom, 
				ajF.get("telefon"), ajF.get("mobil"), 
				ajF.get("eMail"));
		
		this.getLogger().finer(
			"Muntat Contacte " + ajF.getPrefix() + ": " + c);
		
		return c;
	}

	/**
	 * Carrega els controladors
	 * @param prp arxiu de propietats font
	 * @throws GATDExcepcioUsuaris si no es pot donar d'alta un Controlador
	 * @throws GATDExcepcioConfig cas de mancar quelcom
	 */
	private void carregaControladors(Properties prp) throws GATDExcepcioUsuaris {
		this.getLogger().info("------ Carregar Controladors");
		final AjudantPrp aj = new AjudantPrp(prp, "controladors");
		for(int i = 1; i <= aj.getQuants(); ++i) {
			AjudantPrp ajF = aj.nouFill(i);
			Controlador c = new Controlador(
				this.carregaUsuari(ajF), 
				this.carregaClau(ajF),
				this.carregaDI(ajF),
				this.carregaContacte(ajF, null),
				this.carregaAdreça(ajF, null),
				this.carregaIdioma(ajF), 
				this.carregaNumSS(ajF)				
			);
			this.getLogger().finer("Donar d'alta controlador " + c);
			//this.alta(c);
			this.gUsuaris.alta(c);
		}
		this.getLogger().info(
				"Carregat " + aj.getQuants() + " controlador");
	}	
	
	/**
	 * Carrega un número de la seguretat social d'un grup de propietats
	 * @param aj ajudant per accedir al grup de propietats
	 * @return String S = [aj].numSS
	 */
	private String carregaNumSS(AjudantPrp aj) {
		return aj.get("numSS");
	}
	
	/**
	 * Carrega els tècnics
	 * @param prp arxiu de propietats font
	 * @throws GATDExcepcioUsuaris si no es pot donar d'alta un Tècnic
	 * @throws GATDExcepcioConfig cas de mancar quelcom
	 */
	private void carregaTecnics(Properties prp) throws GATDExcepcioUsuaris  {
		this.getLogger().info("------ Carregar Tècnics");
		final AjudantPrp aj = new AjudantPrp(prp, "tecnics");
		for(int i = 1; i <= aj.getQuants(); ++i) {
			AjudantPrp ajF = aj.nouFill(i);
			Tecnic t = new Tecnic(
				this.getEM().find(Seu.class, ajF.get("seu")),
				this.carregaUsuari(ajF), 
				this.carregaClau(ajF),
				this.carregaDI(ajF),
				this.carregaContacte(ajF, null),
				this.carregaAdreça(ajF, null),
				this.carregaIdioma(ajF), 
				this.carregaNumSS(ajF),
				ajF.getBol("actiu")
			);
			this.getLogger().finer("Donar d'alta tècnic " + t);
			//this.alta(t);

			this.gUsuaris.alta(t);
		}
		this.getLogger().info(
				"Carregat " + aj.getQuants() + " tècnics");
	}	
	
	/**
	 * Carrega les peticions
	 * @throws GATDExcepcioConfig cas d'error al llegir les propietats
	 * @throws GATDExcepcioUsuaris si no troba el client o el tècnic
	 */
	private void carregaPeticions(Properties prp) 
	throws GATDExcepcioConfig, GATDExcepcioUsuaris {
		this.getLogger().info("Carregar Peticions");
		final AjudantPrp aj = new AjudantPrp(prp, "peticions");
        for(int i = 1; i <= aj.getQuants(); ++i) {
            AjudantPrp ajF = aj.nouFill(i);
            Client cl = this.gUsuaris.troba(
                    Client.class, ajF.get("client"));
                       
            Peticio p = new Peticio(
              this.carregaDataAlta(ajF),
              this.carregaDataBaixa(ajF),
              cl,
              this.carregaAdreça(ajF, new Adreça(cl.getAdreça())),
              this.carregaContacte(ajF, new Contacte(cl.getContacte())),
              this.carregaDescripcio(ajF),
              this.getEM().find(Tecnic.class, ajF.get("tecnic")),
              Peticio.Estat.valueOf(ajF.get("estat")),
              ajF.getFloat("temps")
            );

            this.alta(p);
            this.carregaQueixa(ajF, p);            
                   
            if(p.getAdreça().getDp().getSeu() == null)
            	throw new GATDExcepcioConfig(
            		"El CP " + p.getAdreça().getDp().getCodi() +
            		" de " + p.getBreu() + " no correspón a cap seu");
            
            this.getLogger().finer("Donada d'alta petició " + p);
            
            assert p.isPendent() || p.isBaixa();
        }
        this.getLogger().info(
                "Carregat " + aj.getQuants() + " peticions");		
		
	}
	
    /**
	 * Carrega una descripció d'un grup de propietats
	 * @param aj ajudant que accedeix al grup de propietats
	 * @return [aj].descripcio
	 */
    private String carregaDescripcio(AjudantPrp aj) {
       return aj.get("descripcio");
    }
    
    /**
     * Carrega a una petició, una queixa construïda d'un grup de
     * propietats "*.queixa.*".
     * <p>
     * Si la propietat *.queixa.dataAlta no existeix, no fa res
     * </p>
     * @param aj ajudant que accedeix al grup de propietats
     * @param p la petició
     * @throws GATDExcepcioConfig si alguna data no està ben especificada
     */
    private void carregaQueixa(AjudantPrp aj, Peticio p) 
    throws GATDExcepcioConfig {
    	AjudantPrp ajF = aj.nouFill("queixa");
    	
    	// La data d'alta es obligatòria
    	Date dataAlta = this.carregaDataAlta(ajF);
    	if(dataAlta != null) {
    		// el constructor lliga la queixa a la petició i al client
    		 new Queixa(
    			p,
    			ajF.get("descripcio"),
    			p.getClient(),
    			dataAlta, 
    			this.carregaDataBaixa(ajF)    			
    			);    		
    	}
    }
    
       


    /**
	 * Carrega una data d'alta d'un grup de propietats
	 * @param aj ajudant que accedeix al grup de propietats
	 * @return la data representada per la propietat [aj].dataAlta,
	 *  en format [any]-[mes]-[dia] [hora (0-23)]:[minuts]
     * @throws GATDExcepcioConfig si la propietat no és en el format indicat
	 */
	private Date carregaDataAlta(AjudantPrp aj) throws GATDExcepcioConfig {
	    return aj.getDate("dataAlta");
	}
	
    /**
     * Carrega una data de baixa d'un grup de propietats
     * @param aj ajudant que accedeix al grup de propietats
     * @return la data representada per la propietat [aj].dataBaixa,
     *  en format [any]-[mes]-[dia] [hora (0-23)]:[minuts]
     * @throws GATDExcepcioConfig si la propietat no és en el format indicat
     */
    private Date carregaDataBaixa(AjudantPrp aj) throws GATDExcepcioConfig {
        return aj.getDate("dataBaixa");
    }	
	
	/**
	 * Ajuda a llegir propietats d'un conjunt d'idem,
	 * caracteritzades per un prefix comú: [prefix].[sufix]
	 * @author Jordi Tejero
	 *
	 */
	private static class AjudantPrp {
		/**
		 * Conjunt de propietats de on llegeix
		 */
		private final Properties prp;
		
		/**
		 * Prefix comú de les propietats que llegeix
		 */
		private final String prefix;

		/**
		 * Constructor
		 * @param prp
		 * @param prefix
		 */
		AjudantPrp(Properties prp, String prefix) {
			super();
			this.prp = prp;
			this.prefix = prefix;
		}
	
		
		/**
		 * Crea un ajudant amb el mateix conjunt de propietats
		 * i un nou prefix comú <code>this.prefix + "." + prefixFill</code>
		 * @param prefixFill part afegida, després del punt, al prefix
		 * @return un nou ajudant F amb  
		 * 	<code>F.prefix == this.prefix + "." + prefixFill</code>
		 */
		AjudantPrp nouFill(String prefixFill) {
			return new AjudantPrp(this.prp, 
					this.prefix + "." + prefixFill); 
		}
		
		/**
		 * Crea un ajudant amb el mateix conjunt de propietats
		 * i un nou prefix comú <code>this.prefix + "." + index</code>
		 * @param index enter que s'afegeix, després del punt, al prefix
		 * @return un nou ajudant F amb  
		 * 	<code>F.prefix == this.prefix + "." + index</code>
		 */
		AjudantPrp nouFill(int index) {
			return this.nouFill(String.valueOf(index)); 
		}		
		
		
		
		/**
		 * Accessor de lectura de l'atribut prefix
		 * @return el valor de l'atribut <code>prefix</code>
		 */
		String getPrefix() {
			return this.prefix;
		}


		/**
		 * Recupera la propietat [this.prefix].[sufix] com a
		 * cadena
		 * @param sufix el sufix
		 * @param omissio el valor per omissió
		 * @return la cadena corresponent a la propietat, o el
		 *  valor per omissió
		 */
		String get(String sufix, String omissio) {
			return this.prp.getProperty(this.prefix + "." + sufix, omissio);
		}
		
		/**
		 * Recupera la propietat [this.prefix].[sufix] com a
		 * cadena
		 * @param sufix el sufix
		 * @return la cadena corresponent a la propietat, o 
		 *  <code>null</code>
		 */
		String get(String sufix) {
			return this.get(sufix, null);
		}
		
		/**
		 * Recupera la propietat [this.prefix].[sufix] com a
		 * enter
		 * @param sufix el sufix
		 * @return l'enter corresponent a la propietat, o
		 *  0 si no existeix
		 */
		int getInt(String sufix) {
			return Integer.parseInt(this.get(sufix, "0"));
		}
		
		/**
		 * Recupera la propietat [this.prefix].quants com a
		 * enter
		 * @return l'enter corresponent a la propietat
		 */
		int getQuants() {
			return this.getInt("quants");
		}
		
		
		/**
		 * Recupera la propietat [this.prefix].[sufix] com a real
		 * @param sufix el sufix
		 * @return el número real representat per la propietat, en
		 *  la codificació estàndard de Java, o 0 si no existeix
		 */
		 float getFloat(String sufix) {
		    return Float.parseFloat(this.get(sufix));		  
		}


		/**
		 * Recupera la propietat [this.prefix].[sufix] com a
		 * boleà
		 * @param sufix el sufix
		 * @return el boleà corresponent a la propietat, o <code>false</code>
		 *  si no existeix
		 */
		boolean getBol(String sufix) {
			return Boolean.parseBoolean(this.get(sufix, "false"));
		}
			
        /**
         * Recupera la propietat [this.prefix].[sufix] com a
         * data. 
         * <p>El valor de la propietat ha de representar una data en format
         * [any]-[mes]-[dia] [hora (0-23)]:[minuts]
         * @param sufix el sufix
         * @return la data representada per la propietat, o <code>null</code>
         *  si no existeix o no representa una data
         * @throws GATDExcepcioConfig si la propietat no és en el format
         *  indicat
         */
        Date getDate(String sufix) throws GATDExcepcioConfig {
            try {
                return Util.parseDate(this.get(sufix));
            } catch (ParseException e) {
                throw new GATDExcepcioConfig(e, "conf_excp_FormatData", 
                		this.get(sufix));
            }
        }		
		
	}
}
