/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.controlador;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.logging.Logger;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;

import edu.uoc.tcfj2ee_gatd.model.comu.GATDExcepcio;
import edu.uoc.tcfj2ee_gatd.model.comu.IMissatgeAmbFormat;


/**
 * Ascendent comú de tots els "managed bean" controladors.
 *
 * <p>
 * Proporciona el "logger" i mètodes per a escriure missatges a les pàgines
 * JSF. 
 * </p>
 *
 * @author Jordi Tejero
 *
 * @param <E> el tipus de les entitats que gestiona
 */
public class EJBControlador {
	
	
	/**
	 * El logger
	 */
	private final Logger logger = Logger.getLogger(this.getClass().getName());
	
	/**
	 * Constructor. Crea el logger
	 *
	 */
	public EJBControlador() {
		/*this.getLogger().finer(
			"Controlador [" + this.getClass().getSimpleName() + "] construit");*/
	}
	
	
	/**
	 * Visualitza una excepció con un missatge d'error associat a un
	 * component
	 * @param e la excepció
	 * @param compId identificador del component al que s'associa el
	 *  missatge. Pot ser <code>null</code>.
	 */
	protected void pintaError(GATDExcepcio e, String compId) {
		
		this.getLogger().fine("Excepció " + e + " al component " + compId);
		
		this.pintaMissatge(FacesMessage.SEVERITY_ERROR,
				this.getMsg(e),
				e.getClass().getSimpleName(),
				compId);
	}
	
	
	
	/**
	 * Visualitza una un missatge informatiu associat a un component
	 * @param clauResum clau a l'arxiu de missatge que dona el resum del missatge
	 * @param clauDetall clau a l'arxiu de missatges que dona el missatge 
	 *  detallat. Pot ser <code>null</code>
	 * @param compId identificador del component al que s'associa el
	 *  missatge. Pot ser <code>null</code>.
	 */
	protected void pintaInfoComp(			
			String clauResum, 
			String clauDetall,
			String compId) {
		
		this.pintaMissatge(FacesMessage.SEVERITY_INFO, 
			this.getMsg(clauResum), 
			this.getMsg(clauDetall), 
			compId);
	}	
	
	/**
	 * Visualitza una un missatge informatiu, no associat a cap component
	 * @param clauResum clau a l'arxiu de missatge que dona el resum del missatge
	 * @param clauDetall clau a l'arxiu de missatges que dona el missatge 
	 *  detallat. Pot ser <code>null</code>
	 */
	protected void pintaInfo(			
			String clauResum, 
			String clauDetall) {
		this.pintaInfoComp(clauResum, clauDetall, null);
	}
	
	/**
	 * Visualitza un missatge informatiu, no associat a cap component, amb
	 * argument addicionals
     * @param clauResum clau a l'arxiu de missatge que dona el resum del missatge
     * @param clauDetall clau a l'arxiu de missatges que dona el missatge 
     *  detallat. Pot ser <code>null</code>
	 * @param aargs el arguments addicionals, que es formatejen segons
	 *  tots dos missatges
	 */
	protected void pintaInfo(String clauResum, 
	        String clauDetall, Object... aargs) {
	    this.pintaMissatge(FacesMessage.SEVERITY_INFO, 
	            this.getMsg(clauResum, aargs), 
	            this.getMsg(clauDetall, aargs), 
	            null);	    
	}
	
	
	/**
	 * Dona el text corresponent a un missatge formatat, muntat
	 * a partir d'una clau a l'arxiu de missatges i d'un seguit
	 * d'arguments addcionals
	 * @param clau la clau del missatge a l'arxiu de missatges de l'aplicació
	 * @param aargs seguit d'arguments addicionals. Pot ser <code>null</code>.
	 * @return
	 * <ul>
	 * <li> 
	 * Si el missatge te arguments addicionals <code>aargs</code>, la cadena 
	 * 	<code>String.format(M, aargs)</code>, on M és l'entrada corresponent
	 *  a <code>clau</code> al "resource bundle" de missatges de 
	 *  l'aplicació, o M = clau si no hi ha tal entrada.
	 * <li>
	 * Si el missatge no té arguments addicionals, l'entrada corresponent
	 *  a <code>clau</code> al "resource bundle" de missatges de 
	 *  l'aplicació, o <code>clau</code> si no hi ha tal entrada. 
	 * </ul> 
	 */
	/*
	private String getMsg(String clau, Object... aargs) {
		FacesContext context = FacesContext.getCurrentInstance();
		
		Application app = FacesContext.getCurrentInstance().getApplication();
	
				 
		String msg = context.getApplication().evaluateExpressionGet(
			context, "#{m['" + clau + "']}" , String.class);
		 
		 if(msg == null)
			 msg = clau;	
		
		 return aargs == null? msg : String.format(msg, aargs);
	}	
	*/
	
	/**
	 * Visualitza un missatge d'error associat a un component
	 * @param resum el resum del missatge
	 * @param detall el missatge detallat. Pot ser <code>null</code>
	 * @param compId identificador del component al que s'associa el
	 *  missatge. Pot ser <code>null</code>.
	 */
	protected void pintaError(
			String resum, 
			String detall,
			String compId) {
		
		this.pintaMissatge(FacesMessage.SEVERITY_ERROR, resum, detall, compId);
	}
	
	/**
	 * Visualitza un missatge d'error associat al component, sense detall
	 * @param resum el resum del missatge
	 * @param compId identificador del component al que s'associa el
	 *  missatge. Pot ser <code>null</code>.
	 */
	protected void pintaError(String resum, String compId) {
		this.pintaError(resum, null, compId);
	}
	/**
	 * Visualitza un missatge no associat a cap component, sense detall
	 * @param resum el resum del missatge
	 */
	protected void pintaError(String resum) {
		this.pintaError(resum, null, null);
	}	
	
	/**
	 * Visualitza una un missatge associat a un component
	 * @param nivell la gravetat del missatge en la escala de Faces
	 * @param resum el resum del missatge
	 * @param detall el missatge detallat. Pot ser <code>null</code>
	 * @param compId identificador del component al que s'associa el
	 *  missatge. Pot ser <code>null</code>.
	 */
	private void pintaMissatge(
			FacesMessage.Severity nivell, 
			String resum, 
			String detall,
			String compId) {
		
		
		 FacesContext context = FacesContext.getCurrentInstance();
		 context.addMessage(compId, 
				 new FacesMessage(nivell, resum, detall)); 
	}
	
	/**
	 * Visualitza una excepció con un missatge d'error no associat a un
	 * component
	 * @param e la excepció
	 */
	protected void pintaError(GATDExcepcio e) {
		this.pintaError(e, null);
	}

	/**
	 * Publica el logger
	 * @return el valor de l'atribut <code>logger</code>
	 */
	 protected Logger getLogger() {
		return this.logger;
	}

    /**
     * Dona el text d'un missatge amb format, a l'idioma de la sessió
     * @param mf el missatge formatat
     * @return els arguments [mf.aargs] formatejats amb
     *  l'idioma de la sessió segons el patró P, on P és l'entrada
     *  a l'arxiu de missatges de l'aplicació, corresponent a 
     *  [mf.clau], o P = [mf.clau] si no hi ha cap 
     *  entrada corresponent.
     */
    String getMsg(IMissatgeAmbFormat mf) {
        return this.getMsg(mf.getClau(), mf.getAargs());
    }

    /**
     * Dona el text d'un missatge amb format, a l'idioma de la sessió
     * 
     * @param clauMsg la clau del missatge a l'arixu de idem
     * @param aargs arguments addicionals. Pot ser <code>null</code>
     * 
     * @return 
     * Si <code>claMsg == null</code>, retorna <code>null</code>.
     * D'altra cas, torna
     * els arguments <code>aargs</code> formatejats amb
     *  l'idioma de la sessió segons el patró P, on P és l'entrada
     *  a l'arxiu de missatges de l'aplicació, corresponent a 
     *  <code>clauMsg</code>, o P = <code>clauMsg</code> si no hi ha cap 
     *  entrada corresponent.
     */
    protected String getMsg(String clauMsg, Object... aargs) {
        
        if(clauMsg == null)
            return null;
        
        FacesContext ctx = FacesContext.getCurrentInstance();
        Locale locSessio = ctx.getViewRoot().getLocale();
        ResourceBundle mb = ResourceBundle.getBundle(
                ctx.getApplication().getMessageBundle(), locSessio);
        
        
        String patro;
        try {
            patro = mb.getString(clauMsg);
         }catch(MissingResourceException e) {
             patro = clauMsg;
         }	         	   
         
        MessageFormat mf = new MessageFormat(patro);
    
       /* this.getLogger().finer(this.getClass().getSimpleName() 
        	+ "#getMsg(" + clauMsg + ", ...) en " + locSessio + " -> " 
        	+ mf.format(aargs));*/
        
        return mf.format(aargs);
    }
    
    /**
     * Missatge amb un argument, que pot ser obtingut des de les pàgines JSF.
     * @param clauMsg la clau del missatge
     * @param arg l'argument
     * @return el missatge formatejat amb l'idioma de la sessió.
     */
    public String getMsg1(String clauMsg, Object arg) {
    	return this.getMsg(clauMsg, arg);
    }
	
}
