/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.controlador.atencio;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import javax.faces.context.FacesContext;

import edu.uoc.tcfj2ee_gatd.configuracio.Configuracio;
import edu.uoc.tcfj2ee_gatd.configuracio.GestorConfiguracio;
import edu.uoc.tcfj2ee_gatd.model.atencio.EMailSenzill;
import edu.uoc.tcfj2ee_gatd.model.atencio.IConstructorEMail;
import edu.uoc.tcfj2ee_gatd.model.atencio.entitats.Tecnic;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Peticio;

/**
 * Constructor dels correus que han de rebre els tècnics.
 * <p>
 * És inclòs al package controlador, perquè fa servir objectes de JSF i els 
 * e-mai que genera contenen vincles cap a pàgines de l'aplicació.
 * </p>
 * @author Jordi Tejero
 *
 */
public class ConstructorEMail implements IConstructorEMail {

	/**
	 * Constructor
	 */
	public ConstructorEMail() {
		super();
	}

	/**
	 * Dona el missatge e-mail que ha de rebre un tècnic quan li és
	 * assignada una petició
	 * 
	 * @param t el tècnic destinatari del missatge
	 * @param p la petició que ha estat assignada al tècnic
	 * @return el missatge dirigit a l'adreça e-mail del tècnic
	 *
	 * @see edu.uoc.tcfj2ee_gatd.model.atencio.IConstructorEMail#getMsgAssignada(edu.uoc.tcfj2ee_gatd.model.atencio.entitats.Tecnic, edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Peticio)
	 */
	@Override
	public EMailSenzill getMsgAssignada(Tecnic t, Peticio p) {
		assert t != null;
		assert p != null;
		assert t.getContacte().geteMail() != null;
		
		Configuracio cfg = GestorConfiguracio.getConfig();
		assert cfg != null;
		
		EMailSenzill email = new EMailSenzill();
		String empresa = this.getMsg(t.getCodiIdioma(), "empr_Nom"); 
		
		String assumpte = this.getMsg(t.getCodiIdioma(),
			"aten_mail_Assignada_assumpte", 
			empresa, 
			p.getReferencia());				
				
		String urlPeti = "http://" + cfg.getServidor();
		if(cfg.getPort() != null)
			urlPeti += ":" + cfg.getPort();
		urlPeti += "/GATD/tecnic.faces?refPeti=" + p.getReferencia();
		
		String text = this.getMsg(t.getCodiIdioma(), 
				"aten_mail_Assignada_text", 
				p.getReferencia(), urlPeti); 
		
		email.getTo().add(t.getContacte().geteMail());
		email.setFrom(cfg.getMailRemitent());
		email.setSubject(assumpte);
		email.getText().append(text);
		
		return email;
	}
	
    /**
     * Dona el text d'un missatge amb format, en l'idioma que s'indiqui 
     * 
     * @param codiIdioma codi ISO de l'idioma
     * @param clauMsg la clau del missatge a l'arixu de idem
     * @param aargs arguments addicionals. Pot ser <code>null</code>
     * 
     * @return 
     * Si <code>claMsg == null</code>, retorna <code>null</code>.
     * D'altra cas, torna
     * els arguments <code>aargs</code> formatejats amb
     *  l'idioma indicat segons el patró P, on P és l'entrada
     *  a l'arxiu de missatges de l'aplicació, corresponent a 
     *  <code>clauMsg</code>, o P = <code>clauMsg</code> si no hi ha cap 
     *  entrada corresponent.
     */
    private String getMsg(String codiIdioma, String clauMsg, Object... aargs) {
        
        if(clauMsg == null)
            return null;
        
        FacesContext ctx = FacesContext.getCurrentInstance();
        Locale locale = new Locale(codiIdioma);        
        ResourceBundle mb = ResourceBundle.getBundle(
                ctx.getApplication().getMessageBundle(), locale);
        
        
        String patro;
        try {
            patro = mb.getString(clauMsg);
         }catch(MissingResourceException e) {
             patro = clauMsg;
         }	         	   
         
        MessageFormat mf = new MessageFormat(patro);

    
        return mf.format(aargs);
    }	

}
