/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.controlador.atencio;

import java.util.logging.Level;

import javax.annotation.PostConstruct;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;

import edu.uoc.tcfj2ee_gatd.controlador.EJBControlador;
import edu.uoc.tcfj2ee_gatd.controlador.usuaris.ControladorSessio;
import edu.uoc.tcfj2ee_gatd.model.atencio.entitats.Tecnic;
import edu.uoc.tcfj2ee_gatd.model.comu.GATDExcepcio;
import edu.uoc.tcfj2ee_gatd.model.peticions.GATDExcepcioPeticions;
import edu.uoc.tcfj2ee_gatd.model.peticions.GestorPeticions;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Client;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Peticio;
import edu.uoc.tcfj2ee_gatd.model.usuaris.GestorUsuaris;


/**
 * Controlador MVC del subsistema d'Atenció. Responsable de la navegació dels 
 * tècnics.
 * @author Jordi Tejero
 *
 */
@ManagedBean(name = "atencio")
@RolesAllowed({"Tecnic"})
@RequestScoped
public class ControladorAtencio 
extends EJBControlador {
	
	/**
	 * El gestor de peticions
	 */
	@EJB
	private GestorPeticions gPeticions;
	
	/**
	 * El gestor d'usuaris
	 */
	@EJB
	private GestorUsuaris gUsuaris;
	
	
	/**
	 * El controlador de sessió que proporciona el tècnic
	 */
	@ManagedProperty(value = "#{sessio}")
	private ControladorSessio sessio;
	
	
	/**
	 * La petició actual. És la que es crea o edita
	 */
	private Peticio peti;
	
	/**
	 * La petició acceptada pel tècnic de la sessió, o una de nova
	 * si no en té cap d'acceptada
	 */
	private Peticio petiAcceptada;
	
	/**
	 * El client actual, que es consulta
	 */
	private Client client;
	
	/**
	 * Les peticions del client actual
	 */
	private Iterable<Peticio> peticionsClient;
	
	/**
	 * El tècnic actual, que es consulta. No és el tècnic de la sessió.
	 */
	private Tecnic tecnic;
	
	/**
	 * Constructor. Inicialitza els atributs.
	 */
	public ControladorAtencio() {
		super();
		this.peti = new Peticio();
		this.petiAcceptada = new Peticio();
		this.client = new Client();
		this.tecnic = new Tecnic();
	
	}
	
	/**
	 * Incialització. Si la petició HTTP tenia un paràmetre amb la referència
	 * de la petició, la carreguem.
	 * <p>
	 * Matusser, però funciona
	 * </p>
	 * 
	 */
	@PostConstruct
	public void ini() {
		// Traiem la referència directament de la petició http
		String ref = this.sessio.getParamHttp(".*refPeti");
		try {
			if(ref != null)
				this.setRefPeti(Long.parseLong(ref));
		}
		catch(Throwable t) {
			this.getLogger().log(Level.SEVERE, 
				"*** ERROR INESPERAT a " + this.getClass().getSimpleName() + "#ini()", 
				t);
		}
	}
	
	
	/**
	 * Permet l'injecció del controlador de sessió
	 * @param sessio el nou valor de l'atribut sessió
	 */
	public void setSessio(ControladorSessio sessio) {
	    this.sessio = sessio;
	}
	
	/**
	 * Establiment de la petició actual a partir de la seva referència.
	 * <p>
	 * Cerca la petició al gestor de persistència. Estableix també la petició
	 * acceptada com la que pugui tenir el tècnic responsable de la petició.
	 * </p>
	 * @param referencia la referència de la petició
	 */
	public void setRefPeti(Long referencia) {
	/*	this.getLogger().finer(this.getClass().getSimpleName() 
			+ "#setRefPeti(" + referencia + ")");*/
		if(referencia != null) {
			this.peti = this.gPeticions.cerca(referencia);
			assert this.peti != null;			
			this.setPetiAcceptada(this.peti.getTecnic().getAcceptada());			
			
			/*this.getLogger().finer(this.getClass().getSimpleName() 
			+ "#setRefPeti(" + referencia + "). Acceptada: " +
				this.getRefAcceptada());*/
			
			this.getLogger().finer(this.getClass().getSimpleName() + 
					"#setRefPeti(" + referencia + "). " +
					"Estat " + this.peti.getEstat().name() + ". " +
					"Versió " + this.peti.getVersio());
			
		}					
	}	
	
	/**
	 * Publica la referència de la petició actual
	 * @return la referència de la petició actual {@link #peti}, o
	 *  <code>null</code> si no és establerta
	 */
	public Long getRefPeti() {
		return this.peti == null? null : this.peti.getReferencia();
	}
	
	/**
	 * Estableix la petició acceptada, cercant el seu número de referència
	 * al gestor de persistència.
	 * <p>
	 * Si el número de referència indicat es <code>null</code>, crea una
	 * nova petició com a petició acceptada.
	 * </p>
	 * @param referencia el número de referència a establir
	 */
	public void setRefAcceptada(Long referencia) {
		this.setPetiAcceptada(referencia == null? 
			null : this.gPeticions.cerca(referencia));
	}
	
	/**
	 * Dona el número de referència de la petició acceptada actual
	 * @return el número de referència de la petició acceptada, o 
	 *  <code>null</code> si no es establerta
	 */
	public Long getRefAcceptada() {
		return this.petiAcceptada == null? 
			null : this.petiAcceptada.getReferencia();
	}
	
	/**
	 * Establiment del client actual a partir del seu nom d'usuari
	 * <p>
	 * Si <code>nomClient</code> no és <code>null</code>, cerca el
	 * client corresponent al gestor de persistència i el desa a
	 * {@link #client}
	 * </p>
	 * @param nomClient el nom d'usuari del client a establir.
	 */
	public void setNomClient(String nomClient) {
		if(nomClient != null) {
			this.getLogger().finer(this.getClass().getSimpleName() 
				+ "#setNomClient(" + nomClient + ")");			
			this.client = this.gUsuaris.cerca(Client.class, nomClient);
			assert this.client != null;
		}
		
	}
	
	/**
	 * Publica el nom d'usuari del client actual
	 */
	public String getNomClient() {
		return this.client.getNomUsuari();
	}
	
	/**
	 * Publica el nom d'usuari del tècnic actual. És el que es vol
	 * consultar, no el de la sessió
	 * @return el nom d'usuari del tècnic actual
	 */
	public String getNomTecnic() {
		return this.tecnic.getNomUsuari();
	}
	
	/**
	 * Estableix el tècnic actual, cercant-lo al gestor de persistència
	 * a partir del seu nom d'usuari
	 * @param nomTecnic el nom d'usuari del tècnic
	 */
	public void setNomTecnic(String nomTecnic) {
		this.tecnic = this.gUsuaris.cerca(Tecnic.class, nomTecnic);
		assert this.tecnic != null;
	}
	
	
	/**
	 * Accessor de lectura de la petició que s'edita
	 * @return el valor de l'atribut <code>peti</code>
	 */
	public Peticio getPeti() {
		/*this.getLogger().finer(this.getClass().getSimpleName() 
			+ "#getPeti() -> " + this.peti.getBreu());*/
		return this.peti;
	}


	/**
	 * Accessor de lectura de l'atribut petiAcceptada
	 * @return el valor de l'atribut <code>petiAcceptada</code>
	 */
	public Peticio getPetiAcceptada() {
	/*	this.getLogger().finer(this.getClass().getSimpleName() 
			+ "#getPetiAcceptada() - > " + 
			(this.petiAcceptada == null? null : this.petiAcceptada.getBreu()) );*/
		return this.petiAcceptada;
	}

	/**
	 * Estableix la petició acceptada. Si l'argument és <code>null</code>,
	 * en crea una de nova.
	 * @param petiAcceptada
	 */
	private void setPetiAcceptada(Peticio petiAcceptada) {
		this.petiAcceptada = petiAcceptada == null? 
			new Peticio() : petiAcceptada;
	}
	
	/**
	 * Accessor de lectura de l'atribut client
	 * @return el valor de l'atribut <code>client</code>
	 */
	public Client getClient() {
		return this.client;
	}

	/**
	 * Publica el tècnic actual, per a la seva consulta
	 * @return el valor de l'atribut <code>tecnic</code>
	 */
	public Tecnic getTecnic() {
		return this.tecnic;
	}

	/**
	 * Dona les peticions pendents del tècnic de la sessió.
	 * Actualitza <code>petis</code>.
	 * @return iterable que recorre les peticions pendents
	 */
	public Iterable<Peticio> getPeticions()  {
		assert this.sessio.getNomUsuari() != null;

		/*this.getLogger().finer("ControladorAtencio#getPeticions(). " +
				"Tecnic " + this.sessio.getNomUsuari());*/
		
		return this.gPeticions.llistaPetisPendentsTecnic(
			this.sessio.getNomUsuari());
	}
	
	/**
	 * Dona totes les peticions del client actual, pendents i tancades
	 * @return
	 */
	public Iterable<Peticio> getPeticionsClient() {
		if(this.peticionsClient == null) try {
			this.peticionsClient = 
				this.gPeticions.llistaPeticionsClient(this.client);
		}
		catch(GATDExcepcioPeticions e) {
			this.pintaError(e);
		}
		return this.peticionsClient;
	}


	/**
	 * <i>Forward</i> cap a la pàgina adient per a atendre o consultar la 
	 * petició actual, en funció de si és responsabilitat del tècnic de la
	 * sessió, de l'estat de la petició i de si el tècnic de la sessió en té 
	 * alguna altra d'acceptada.
	 * <p>
	 * Emulació del comportament del "commandLink", que no és operatiu
	 * al mòbil triat com a referència.
	 * </p>
	 */
	public void vesAtendre() {
		
		this.getLogger().finer(this.getClass().getSimpleName() + 
			"#vesAtendre(). " + this.peti.getBreu() + 
			" en estat " + this.peti.getEstat());
		 String pag = null;
		 
		 assert this.peti != null;
		 assert this.peti.isAlta();
		 
		 if(!this.peti.isPendent()
		 || !this.peti.getTecnic().equals(this.sessio.getUsuari()))
			 pag = "/contingut/tecnics/consultaPeticio.faces";
		 else if(this.peti.isAcceptada())
			 pag = "/contingut/tecnics/peticioAcceptada.faces";
		 else if(this.petiAcceptada == null || !this.petiAcceptada.isAlta())
			 pag = "/contingut/tecnics/peticioAssignada.faces";
		 else
			 pag = "/contingut/tecnics/peticionsAssignadaAcceptada.faces";
		 
		 this.sessio.vesA(pag);
	 }
	
	
	/**
	 * Finalitza la petició actual i passa al llistat de peticions
	 * del tècnic de la sessió.
	 * <p>
	 * Si no s'hi ha posat el temps invertit, error.
	 * </p>
	 * 
	 * @return la pàgina del llistat de peticions del tècnic en cas de tenir
	 *  èxit, o <code>null</code> si es produeix un error al guardar els canvis.
	 */
	public String finalitza() {
		/*assert this.peti.isAlta();
		assert this.peti.isAcceptada();*/
		
		this.getLogger().finer(
			"ControladorAtencio#finalitza(). Ref =" + this.peti.getReferencia());
					
		boolean exit = false;
		
		try {
			this.gPeticions.finalitza(this.sessio.getNomUsuari(), this.peti);
			exit = true;
		} catch (GATDExcepcio e) {
			this.pintaError(e);
		}
		
		return exit? "atenPeticions.xhtml" : null;
	}	
	
	/**
	 * El tècnic de la sessió accepta o finalitza la petició actual, en 
	 * funció del seu estat.
	 * <ol> 
	 * <li>
	 * Si la petició actual és en estat <code>ACCEPTADA</code>, la
	 * finalitza i navega cap al llistat de peticions.
	 * </li>
	 * <li>
	 * Si la petició actual és en estat <code>ASSIGNADA</code>, l'accepta.
	 * Si a més a més, el tècnic de la sessió tenia una altra petició
	 * acceptada, finalitza aquesta darrera.
	 * Navega cap al llistat de peticions pendents del tècnic de la sessió. 
	 * </li>
	 * </ol>
	 * <p>
	 * En els casos en que es finalitza una petició, si no s'ha indicat
	 * el temps emprat, escriu un missatge d'error i resta a la mateixa
	 * pàgina.
	 * </p>
	 * 
	 * @return La URL de la fitxa de la petició acceptada
	 */
	public String acceptaFinalitza() {
		assert this.peti != null;
		this.getLogger().finer(this.getClass().getSimpleName() 
			+ "#acceptaFinalitza(). " +
			this.peti.getBreu() + " en estat " + this.peti.getEstat());
		assert this.peti.isAlta();		
		
		String anar = null;

		
		try {
			if(this.petiAcceptada != null && this.petiAcceptada.isAlta())
				this.gPeticions.finalitza(this.sessio.getNomUsuari(), 
						this.petiAcceptada);		
			if(this.peti.isAcceptada()) {
					this.gPeticions.finalitza(this.sessio.getNomUsuari(), 
						this.peti);
					anar = "atenPeticio";
			}
			else {
				assert this.peti.isAssignada();
				this.gPeticions.accepta(this.sessio.getNomUsuari(), this.peti);
				anar = "atenPeticions";
			}
		} catch (GATDExcepcio e) {
			this.pintaError(e);
		}
		
		return anar;
	}
	
		
	
	/**
	 * Navegació web. Anar a la pàgina que consulta el client actual
	 * @return la pàgina
	 */
	public String consultaClient() {
		return "consultaClient";
	}

	

}
