/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.controlador.peticions;


import java.sql.Timestamp;

import edu.uoc.tcfj2ee_gatd.configuracio.GestorConfiguracio;
import edu.uoc.tcfj2ee_gatd.controlador.FiltreAbs;
import edu.uoc.tcfj2ee_gatd.model.comu.Util;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Peticio;

/**
 * FiltreAbs de peticions. Consta d'un mètode que retorna una query i d'una serie 
 * d'atributs que controlen com serà.
 * @author Jordi Tejero
 *
 */
 public class FiltrePeticions extends FiltreAbs {
	
	/**
	 * Nom d'usuari del Client al que corresponen les peticions que cerquem. Si 
	 * <code>null</code>, no s'imposa la condició.
	 */
	private String nomClient;
	
	/**
	 * Nom d'usuari del tècnic responsable de les peticions que cerquem. Si 
	 * <code>null</code>, no s'imposa la condició.
	 */
	private String nomTecnic;
	
	
	/**
	 * Número de la referència que es cerca. Si <code>null</code>, no
	 * s'imposa la condició.
	 */
	private Long referencia;
	
	/**
	 * Indica si s'han de mostrar o no les peticions en estat ANULLADA i 
	 * FINALITZADA
	 */
	private boolean tambeTancades;
	
	/**
	 * Número del districte postal de les peticions que es cerquen. Si
	 * es <code>null</code>, no s'imposa la condició 
	 */
	private String cp;

	/**
	 * Codi de la seu a la que correspón la petició
	 */
	private String codiSeu;
	
	/**
	 * Mostrar sols les peticions amb una queixa no anul·lada?
	 */
	private boolean solsQueixades;
	
	/**
	 * Mostrar sols les peticions amb alerta de demora?
	 */
	private boolean solsDemorades;

	
	/**
	 * Constructor
	 */
	public FiltrePeticions() {
		super();		
	}
	
	
	/* Elimina tots els atributs corresponents a criteris de filtrat. La
	 * sentència JPQL trobarà totes les peticions.
	 * @see edu.uoc.tcfj2ee_gatd.controlador.FiltreAbs#ini()
	 */
	@Override
	protected void ini() {
		this.nomClient = null;
		this.nomTecnic = null;
		this.referencia = null;
		this.tambeTancades = false;
		this.cp = null;
		this.codiSeu = null;
		this.solsQueixades = false;
		this.solsDemorades = false;
	}



	/**
	 * Accessor de lectura de l'atribut client
	 * @return el valor de l'atribut <code>client</code>
	 */
	public String getNomClient() {
		return this.nomClient;
	}

	/**
	 * Accessor d'escriptura a l'atribut client
	 * @param nomClient el nou valor de l'atribut client
	 */
	public void setNomClient(String nomClient) {
		this.nomClient = nomClient;
	}

	/**
	 * Accessor de lectura de l'atribut nomTecnic
	 * @return el valor de l'atribut <code>nomTecnic</code>
	 */
	public String getNomTecnic() {
		return this.nomTecnic;
	}

	/**
	 * Accessor d'escriptura a l'atribut nomTecnic
	 * @param nomTecnic el nou valor de l'atribut nomTecnic
	 */
	public void setNomTecnic(String nomTecnic) {
		this.nomTecnic = nomTecnic;
	}

	/**
	 * Accessor de lectura de l'atribut referencia
	 * @return el valor de l'atribut <code>referencia</code>
	 */
	public Long getReferencia() {
		return this.referencia;
	}


	
	/**
	 * Accessor d'escriptura a l'atribut referencia
	 * @param referencia el nou valor de l'atribut referencia
	 */
	public void setReferencia(Long referencia) {
		this.referencia = referencia;
	}

	/**
	 * Accessor de lectura de l'atribut tambeTancades
	 * @return el valor de l'atribut <code>tambeTancades</code>
	 */
	public boolean isTambeTancades() {
		return this.tambeTancades;
	}

	/**
	 * Accessor d'escriptura a l'atribut tambeTancades
	 * @param tambeTancades el nou valor de l'atribut tambeTancades
	 */
	public void setTambeTancades(boolean tambeTancades) {
		this.tambeTancades = tambeTancades;
	}

	/**
	 * Accessor de lectura de l'atribut cp
	 * @return el valor de l'atribut <code>cp</code>
	 */
	public String getCp() {
		return this.cp;
	}

	/**
	 * Accessor d'escriptura a l'atribut cp
	 * @param cp el nou valor de l'atribut cp
	 */
	public void setCp(String cp) {
		this.cp = cp;
	}
	
	/**
	 * Accessor de lectura de l'atribut codiSeu
	 * @return el valor de l'atribut <code>codiSeu</code>
	 */
	public String getCodiSeu() {
		return this.codiSeu;
	}


	/**
	 * Accessor d'escriptura a l'atribut codiSeu
	 * @param codiSeu el nou valor de l'atribut codiSeu
	 */
	public void setCodiSeu(String codiSeu) {
		this.codiSeu = codiSeu;
	}


	/**
	 * Accessor de lectura de l'atribut solsQueixades
	 * @return el valor de l'atribut <code>solsQueixades</code>
	 */
	public boolean isSolsQueixades() {
		return this.solsQueixades;
	}


	/**
	 * Accessor d'escriptura a l'atribut solsQueixades
	 * @param solsQueixades el nou valor de l'atribut solsQueixades
	 */
	public void setSolsQueixades(boolean solsQueixades) {
		this.solsQueixades = solsQueixades;
	}


	/**
	 * Accessor de lectura de l'atribut solsDemorades
	 * @return el valor de l'atribut <code>solsDemorades</code>
	 */
	public boolean isSolsDemorades() {
		return this.solsDemorades;
	}


	/**
	 * Accessor d'escriptura a l'atribut solsDemorades
	 * @param solsDemorades el nou valor de l'atribut solsDemorades
	 */
	public void setSolsDemorades(boolean solsDemorades) {
		this.solsDemorades = solsDemorades;
	}


	/**
	 * La sentència JPQL de selecció, resultant d'aplicar les restriccions
	 * degudes als atributs
	 * @return cadena SELECT sobre la taula Peticio
	 */
	@Override
	public String getJPQL() {	
		this.netejaJPQL();
		this.posaSelect("SELECT p FROM Peticio p");
		
		// Restriccions típiques dels clients
		this.posaAnd("p.client.nomUsuari", this.nomClient);
		if(this.referencia != null)
			this.posaAnd("p.referencia = " + this.referencia);
		if(!this.isTambeTancades()) {
			this.posaAndNo("p.estat", Peticio.Estat.ANULLADA);
			this.posaAndNo("p.estat", Peticio.Estat.FINALITZADA);
		}

		this.posaAnd("p.adreça.dp.codi", this.cp);
		this.posaAnd("p.adreça.dp.seu.codi", this.codiSeu);
		
		
		// restriccions típiques dels tècnics
		this.posaAnd("p.tecnic.nomUsuari", this.nomTecnic);
		if(this.solsQueixades)
			this.posaAnd(
				"p.queixa.dataAlta IS NOT NULL AND p.queixa.dataBaixa IS NULL");
		if(this.solsDemorades) {
			String condicio;
			Timestamp tallAtencio = new Timestamp(System.currentTimeMillis() - 
				(long)(GestorConfiguracio.getConfig().getLlindarAlertaAtencio() 
				* 60 * 60 * 1000));
			Timestamp tallFi = new Timestamp(System.currentTimeMillis() - 
				(long)(GestorConfiguracio.getConfig().getLlindarAlertaFi() 
							* 60 * 60 * 1000));
			condicio = 
			"(p.estat = " + this.getEnumStr(Peticio.Estat.ASSIGNADA) +
			" AND p.dataAlta < '" + tallAtencio.toString() + "'" +
			")" + Util.LS +
			"OR (" + Util.LS +
			"p.estat = " + this.getEnumStr(Peticio.Estat.ACCEPTADA) +
			" AND p.dataAlta < '" + tallFi + "'" +
			")";			
		
			this.posaAnd(condicio);
		}
		
		this.getLogger().finer("FiltrePeticions#getJPQL() ->" + Util.LS + 
				this.getSentencia());
		
		return this.getSentencia();
	}	
	
}
