/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.controlador.usuaris;

import javax.annotation.security.RolesAllowed;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.RequestScoped;

import edu.uoc.tcfj2ee_gatd.controlador.FiltreAbs;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Client;
import edu.uoc.tcfj2ee_gatd.model.usuaris.GATDExcepcioUsuaris;


/**
 * Controlador MVC, responsable de la navegació entre les pàgines 
 * d'administració d'un únic client. 
 * Per als usuaris de classe <code>Client</code>.
 * @author Jordi Tejero
 *
 */
@ManagedBean(name = "client")
@RequestScoped
@RolesAllowed({"Client"})
public class ControladorClient extends ControladorUsuaris<Client> {
	
	/**
	 * Constructor. Carrega l'usuari de la sessió, si s'ha validat
	 * @throws GATDExcepcioUsuaris cas d'error intern
	 */
	public ControladorClient() throws GATDExcepcioUsuaris {
		super(Client.class);	
		
	}

	
	/**
	 * Carrega l'usuari de la sessió
	 */
	public void carrega() {
		if(this.getSessio() != null && this.getSessio().getNomUsuari() != null)
			super.carrega(this.getSessio().getNomUsuari());		
	}
	
	/**
	 * Mira de donar d'alta el client. Si ja hi havia un client
	 * amb el mateix nom, mostra un missatge d'error i retorna <code>null</code>.
	 * 
	 * @return la URL de la pàgina de login o <code>null</code> si no s'ha 
	 *  creat l'usuari client.
	 */
	public String alta() {

		return super.fesAlta()? "login" : null;
	}

	/* No ens cal filtre perquè els usuaris Client no disposen de llistats
	 * de clients
	 * @see edu.uoc.tcfj2ee_gatd.controlador.usuaris.ControladorUsuaris#getFiltre()
	 */
	@Override
	public FiltreAbs getFiltre() {
		
		return null;
	}
		
	
}
