/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.controlador.usuaris;

import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.security.Principal;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Pattern;

import javax.annotation.PostConstruct;
import javax.annotation.security.PermitAll;
import javax.annotation.security.RolesAllowed;
import javax.ejb.EJB;
import javax.faces.application.Application;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;
import javax.faces.component.UIViewRoot;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import edu.uoc.tcfj2ee_gatd.controlador.EJBControlador;
import edu.uoc.tcfj2ee_gatd.model.atencio.entitats.Tecnic;
import edu.uoc.tcfj2ee_gatd.model.comu.Util;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Client;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Controlador;
import edu.uoc.tcfj2ee_gatd.model.usuaris.GestorUsuaris;
import edu.uoc.tcfj2ee_gatd.model.usuaris.entitats.Usuari;


/**
 * Controlador MVC, responsable de la gestió de les sessions.
 * <p>
 * Implementa el fi de sessió, manté el "locale" de les pàgines JSF
 * i proporciona l'usuari a qui correspon la sessió. També ofereix
 * una funció de "forward".
 * </p>
 * <p>
 * Conté fragments de codi trets del llibre "Beginning JSF™ 2 APIs and 
 * JBoss® Seam" de Kent Ka Iok Tong. Ed. Apress, maig 2009.
 * </p>
 * <p>
 * Per a la detecció de l'ús d'un micro-nagegador, es fa servir el mètode
 * proporcionat per http://detectmobilebrowser.com/
 * </p>
 * @author Jordi Tejero
 * @author Kent Ka Iok Tong
 * @author http://detectmobilebrowser.com/
 */
@ManagedBean(name = "sessio")
@RequestScoped
@RolesAllowed({"Tecnic", "Client", "Controlador"})
public class ControladorSessio extends EJBControlador {
	
	/**
	 * Nom de l'atribut de la sessió HTTP on es guarda el codi de 
	 * l'idioma
	 */
	public static final String ATRIBUT_IDIOMA = "GATD.codiIdioma";
	
	/**
	 * El gestor d'usuaris
	 */
	@EJB
	private GestorUsuaris gUsuaris;
	
	/**
	 * La sessió http
	 */
	@ManagedProperty(value = "#{session}")
	private HttpSession sessioHttp; 
	
	/**
	 * Historial de navegació de la sessió
	 */
	@ManagedProperty(value = "#{historial}")
	private HistorialSessio historialSessio;
	
	/**
	 * La petició http
	 */
	@ManagedProperty(value = "#{request}")
	private HttpServletRequest peticioHttp;
	
	
	/**
	 * El logger
	 */
	private final Logger logger = 
		Logger.getLogger(this.getClass().getCanonicalName());
	
	/**
	 * Permet la injecció de la sessió http
	 * @param sessioHttp la sessió http
	 */
	public void setSessioHttp(HttpSession httpSession) {
		this.sessioHttp = httpSession;
	}
	
	/**
	 * Permet la injecció de la petició HTTP
	 * @param peticioHttp el nou valor de l'atribut peticioHttp
	 */
	public void setPeticioHttp(HttpServletRequest peticioHttp) {
		this.peticioHttp = peticioHttp;
	}

	/**
	 * Publica la <i>query string</i> de la petició HTTP
	 * @return la "query sting", o <code>null</code> si no hi ha
	 *  petició HTTP.
	 */
	String getQueryString() {
		return this.peticioHttp == null? null : this.peticioHttp.getQueryString();
	}

	/**
	 * Permet la injecció de l'historial de la sessió.
	 * @param historialSessio el nou valor de l'atribut historialSessio
	 */
	public void setHistorialSessio(HistorialSessio historialSessio) {
		this.historialSessio = historialSessio;
	}

	/**
	 * Afegeix una pàgina a l'historial 
	 * @param via el "path" de la pàgina
	 * @param qs la "query string" de la pàgina
	 */
	void afegeixHistorial(String via, String qs) {
		this.historialSessio.avança(via, qs);
	}
	
	/**
	 * L'usuari a qui pertany la sessió, o <code>null</code> si no s'ha validat
	 */
	private Usuari usuari;
	
	/**
	 * Inicialització. Determina l'usuari de la sessió HTTP i estableix
	 * el seu idioma per a l'interficie.
	 */
	@PostConstruct
	public void ini() {
		Principal pral = this.peticioHttp.getUserPrincipal();
		
		//Logging
		if(this.getLogger().isLoggable(Level.FINER)) {
			this.getLogger().finer(
					"ControladorSessio#ini()." + Util.LS +
					"UserAgent: " +
					this.peticioHttp.getHeader("User-Agent") + ". " + Util.LS +					
					"ContextPath: " +
					this.peticioHttp.getContextPath() + ". " + Util.LS +
					"ServletPath: " + 
					this.peticioHttp.getServletPath() + ". " + Util.LS +					
					"Principal: " + pral + ". " + Util.LS +
					"QS: " + this.peticioHttp.getQueryString());			
			Enumeration<String> pars = this.peticioHttp.getParameterNames();
			int i;
			while(pars.hasMoreElements()) {
				String par = pars.nextElement();
				this.getLogger().finer("Paràmetre [" + par + "]");
				i = 0;
				for(String val : this.peticioHttp.getParameterValues(par))
					this.getLogger().finer("Valor " + (++i) + ": " + val);
			}
		}
		
		if(pral == null)
		    this.usuari = null;
		else {		    
		    this.usuari = this.gUsuaris.cerca(Usuari.class, pral.getName());
		    /*
		    this.getLogger().finer(
		    	"ControladorSessio#ini Usuari " +
		    	"[" + this.usuari.getClass().getSimpleName() + "] " +		        
		        this.usuari.getNomUsuari());
		    */
		    this.changeLangCode(this.usuari.getCodiIdioma());
		}
	}
	
	/**
	 * Dona la pàgina visitada abans de l'actual, o <code>null</code> si no hi 
	 * ha cap de prèvia
	 * @return la URI de la pàgina prèvia visitada, sense URL-codificar, 
	 *  o <code>null</code>
	 */
	public String pagPrevia() {
		//tornar a la pàgina prèvia de l'historial
		return this.decodeURL(this.historialSessio.getURIPrevia());
	}
	
	/**
	 * Dona la darrera pàgina visitada a l'historial, o <code>null</code> si no 
	 * n'hi ha cap
	 * @return la URI de la darrera pàgina visitada, sense URL-codificar
	 */
	public String pagDarrera() {
		return this.decodeURL(this.historialSessio.getURIDarrera());
	}
	
	/**
	 * Descodifcació amb control d'errors d'una URL amb UTF-8
	 * @param url la URL codificada
	 * @return la URL descodificada, o la mateixa si ha ha hagut un error
	 */
	private String decodeURL(String url) {
		String retorn;
		try {
			retorn = URLDecoder.decode(url, "utf-8");
		} catch (UnsupportedEncodingException e) {
			this.getLogger().log(Level.SEVERE, "Descodificació de URL amb UTF-8", e);
			retorn = url;
		}
		
		return retorn;
	}

	/**
	 * Dona els valors d'un paràmetre de la petició HTTP, donat pel seu nom
	 * @param nomParam el nom del paràmetre de la petició HTTP
	 * @return array amb els valors del paràmetre, o <b>null</b> si el
	 *  paràmetre no és a la petició HTTP
	 */
	public String[] getParamsHttp(String nomParam) {
		return this.peticioHttp == null? 
			null : this.peticioHttp.getParameterValues(nomParam);
	}
	
	/**
	 * Dona el valor del primer paràmetre de la petició http, que encaixi
	 * amb una expressió regular
	 * @param rexpParam cadena amb la expressió regular, R
	 * @return el valor del primer paràmetre P que encaixi amb R, o <code>null</code>
	 *  si no n'hi ha cap.
	 */
	public String getParamHttp(String rexpParam) {
		if(this.peticioHttp == null)
			return null;
		
		Pattern ptnParam = Pattern.compile(rexpParam);
		Enumeration<String> np = this.peticioHttp.getParameterNames();
		while(np.hasMoreElements()) {
			String nomParam = np.nextElement();
			if(ptnParam.matcher(nomParam).matches())
				return this.peticioHttp.getParameter(nomParam);
		}
		
		return null;
	}
	
	/**
	 * Diu si la visualització s'ha de fer amb un micro-navegador.
	 * <p>
	 * És el cas dels usuaris de classe <code>Tecnic</code> i de tots
	 * aquells que facin servir un "user-agent" que es puguin reconèixer com a
	 * dispositius mòbil
	 *</p>
	 * @return <code>true</code> si l'usuari de la sessió és un tècnic o
	 *  si fa servir un "user agent" propi de mòbil
	 */
	@PermitAll
	public boolean isMicroNav() {

		boolean microNav =
		   this.getUsuari() != null && this.getUsuari() instanceof Client;

		if(!microNav) {
			/*
			 * Detecció de mòbil pel "user agent". 
			 * Font: http://detectmobilebrowser.com/
			 */
			String ua=this.peticioHttp.getHeader("User-Agent").toLowerCase();
			microNav = ua.matches(
				".*(android|avantgo|blackberry|blazer|compal|elaine|fennec|hiptop|" +
				"ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile|o2|" +
				"opera m(ob|in)i|palm( os)?|p(ixi|re)\\/|plucker|pocket|psp|" +
				"smartphone|symbian|treo|up\\.(browser|link)|vodafone|wap|" +
				"windows ce; (iemobile|ppc)|xiino).*"
			) 
			||ua.substring(0,4).matches(
				"1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|" +
				"ac(er|oo|s\\-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|" +
				"ar(ch|go)|as(te|us)|attw|au(di|\\-m|r |s )|avan|be(ck|ll|nq)|" +
				"bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw\\-(n|u)|c55\\/|capi|ccwa|" +
				"cdm\\-|cell|chtm|cldc|cmd\\-|co(mp|nd)|craw|da(it|ll|ng)|dbte|" +
				"dc\\-s|devi|dica|dmob|do(c|p)o|ds(12|\\-d)|el(49|ai)|em(l2|ul)|" +
				"er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(\\-|_)|g1 u|g560|gene|" +
				"gf\\-5|g\\-mo|go(\\.w|od)|gr(ad|un)|haie|hcit|hd\\-(m|p|t)|hei\\-|" +
				"hi(pt|ta)|hp( i|ip)|hs\\-c|ht(c(\\-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|" +
				"i\\-(20|go|ma)|i230|iac( |\\-|\\/)|ibro|idea|ig01|ikom|im1k|inno|" +
				"ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\\/)|klon|kpt |" +
				"kwc\\-|kyo(c|k)|le(no|xi)|" +
				"lg( g|\\/(k|l|u)|50|54|e\\-|e\\/|\\-[a-w])|libw|lynx|m1\\-w|m3ga|" +
				"m50\\/|ma(te|ui|xo)|mc(01|21|ca)|m\\-cr|me(di|rc|ri)|mi(o8|oa|ts)|" +
				"mmef|mo(01|02|bi|de|do|t(\\-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|" +
				"n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)\\-|" +
				"on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|" +
				"pan(a|d|t)|pdxg|pg(13|\\-([1-8]|c))|phil|pire|pl(ay|uc)|pn\\-2|" +
				"po(ck|rt|se)|prox|psio|pt\\-g|qa\\-a|qc(07|12|21|32|60|\\-[2-7]|" +
				"i\\-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\\/|" +
				"sa(ge|ma|mm|ms|ny|va)|sc(01|h\\-|oo|p\\-)|sdk\\/|se(c(\\-|0|1)|" +
				"47|mc|nd|ri)|sgh\\-|shar|sie(\\-|m)|sk\\-0|sl(45|id)|sm(al|ar|b3|" +
				"it|t5)|so(ft|ny)|sp(01|h\\-|v\\-|v )|sy(01|mb)|t2(18|50)|" +
				"t6(00|10|18)|ta(gt|lk)|tcl\\-|tdg\\-|tel(i|m)|tim\\-|t\\-mo|" +
				"to(pl|sh)|ts(70|m\\-|m3|m5)|tx\\-9|up(\\.b|g1|si)|utst|v400|v750|" +
				"veri|vi(rg|te)|vk(40|5[0-3]|\\-v)|vm40|voda|vulc|" +
				"vx(52|53|60|61|70|80|81|83|85|98)|w3c(\\-| )|webc|whit|" +
				"wi(g |nc|nw)|wmlb|wonu|x700|xda(\\-|2|g)|yas\\-|your|zeto|zte\\-"
			);
		}
		
		this.getLogger().finer(this.getClass().getSimpleName() + 
			"#isMicroNav() -> " + microNav);
		
		return microNav;
	}
	
	
	/**
	 * Entrada en funció del role de l'usuari.
	 * <p>
	 * Si l'usuari de la sessió s'ha validat, navega cap a la
	 * seva pàgina d'entrada. Si no s'ha validat, no fa res.
	 * </p>
	 * @param refPeti referència de la petició a consultar. Sols pot ser 
	 *  diferent de <code>null</code> en el cas dels tècnics 
	 */
	@PermitAll
	public void entra() {
		String pag = null;
		
		this.getLogger().finer(this.getClass().getSimpleName() + "#entra() " + 
			this.getNomUsuari());
		
		if(this.usuari == null)
			{/* no res */}
		else if(this.usuari instanceof Tecnic)
			pag = "/contingut/tecnics/atenPeticions.faces";
		else if(this.usuari instanceof Client)
			pag = "/contingut/clients/administraPeticions.faces";
		else if(this.usuari instanceof Controlador)
			pag = "/contingut/controladors/administraTecnics.faces";
		
		if(pag != null) 
			this.vesA(pag);
					
	}	

	
	/**
	 * Implementació de "forward".
	 * Navega cap a la pàgina que s'indiqui. Cas d'error, simplement 
	 * l'enregistra
	 * @param pag la pàgina a on anar
	 */
	public void vesA(String pag) {
		this.getLogger().finer(this.getClass().getSimpleName() + 
			"#vesA(" + pag + ")");

		try {
			FacesContext.getCurrentInstance().getExternalContext().dispatch(pag);
		} catch (Exception e) {
			this.getLogger().log(Level.SEVERE, 
				"ControladorSessio#vesA(" + pag + "). ** ERROR.", 
				e);
		} 				
	}
	
	/**
	 * Implementació de "redirect".
	 * Redirecció cap a la pàgina que s'indiqui. Cas d'error, simplement 
	 * l'enregistra
	 * @param pag la pàgina a on anar
	 */
	public void redireccio(String pag) {
		this.getLogger().finer(this.getClass().getSimpleName() + 
			"#redirect(" + pag + ")");

		FacesContext context = FacesContext.getCurrentInstance();
		HttpServletResponse response =
		(HttpServletResponse)context.getExternalContext().getResponse();
	
		
		try {
			response.sendRedirect(pag);
		} catch (Exception e) {
			this.getLogger().log(Level.SEVERE, 
				"ControladorSessio#redirect(" + pag + "). ** ERROR.", 
				e);
		} 				
	}	
	
	/**
	 * Finalitza la sessió. Navega fins a la pàgina de comiat.
	 * @return la pàgina pública de comiat
	 */
	public String logOut() {
		
		this.logger.finer("Logout");

		
		String anar = "/contingut/public/comiat";
		
		try {			
			this.sessioHttp.invalidate();	     			
			// Nova sessió amb la que l'usuari pot tornar a entrar, si vol
			this.sessioHttp = this.peticioHttp.getSession();
		}
		catch(Exception e) {
			this.getLogger().log(Level.SEVERE, "ERROR fent logout", e);
		}		
	     
	    return anar;
	    
	}
	
	/**
	 * Obtenció de la petició HTTP actual
	 * @return la petició HTTP actual
	 */
	/*
	private HttpServletRequest getPeticioHttp() {
		return (HttpServletRequest)FacesContext.getCurrentInstance()
			.getExternalContext().getRequest();
	}
	*/

	/**
	 * Dona una llista de "select items" amb els codis de llenguatge
	 * disponibles
	 * @return la llista de <code>SelectItem</code> amb els llenguatges, amb
	 *  llenguatge per omissió en primer lloc.
	 * @author Kent Ka Iok Tong
	 */
	public List<SelectItem> getCodisIdiomes() {
	    List<SelectItem> items = new ArrayList<SelectItem>();
	    Application app = FacesContext.getCurrentInstance().getApplication();
	    
	    // Primer l'idioma per omissió
	    Locale locale = app.getDefaultLocale();
	    String display = locale.getDisplayName(locale);	    
	    items.add(new SelectItem(locale.toString(), display));
	    
	    // Després, la resta d'idiomes
	    Iterator<Locale> iter = app.getSupportedLocales();
	    while (iter.hasNext()) {
	        locale = iter.next();
	        display = locale.getDisplayName(locale);
	        items.add(new SelectItem(locale.toString(), display));
	    }
	    return items;
	}
	

   /**
     * Canvia el codi d'idioma de la sessió i de la vista Faces 
     * <p>
	 * Codi tret del llibre "Beginning JSF™ 2 APIs and 
	 * JBoss® Seam" de Kent Ka Iok Tong. Ed. Apress, maig 2009.
	 * </p>
	 * @param langCode el nou codi d'idioma
	 * @author Kent Ka Iok Tong
	 * @author Jordi Tejero
	 */
	private void changeLangCode(String langCode) {
	   /* HttpSession session = (HttpSession) FacesContext.getCurrentInstance()
                .getExternalContext().getSession(false);
        if (session != null) {
            session.setAttribute("multilang.langCode", langCode);
        }
        }*/
	   
       if (this.sessioHttp != null) 
           this.sessioHttp.setAttribute(ControladorSessio.ATRIBUT_IDIOMA, 
        		  langCode);

        UIViewRoot viewRoot = FacesContext.getCurrentInstance().getViewRoot();
        if(viewRoot != null) {
	        viewRoot.setLocale(new Locale(langCode));
	        this.getLogger().finer(
	        		"ContoradorSessio#changeLangCode(" + langCode + "). Fet");
        }
    }
	
	/**
	 * Si l'usuari que s'indica és el de la sessió, canvia l'idioma de la
	 * interfície per a que coincideixi amb el de l'usuari
	 * @param u l'usuari
	 * @return <code>true</code> sols si s'ha canviat efectivament l'idioma
	 *  de la interfície 
	 */
	boolean canviaIdiomaUsuari(Usuari u) {
		boolean canvi = false;
		if(this.usuari != null && this.usuari.equals(u)
			&& (this.usuari.getCodiIdioma() == null 
			    || !this.usuari.getCodiIdioma().equals(u.getCodiIdioma()))
		) {
			canvi = true;
			this.changeLangCode(u.getCodiIdioma());
		}
		
		return canvi;
	}
	
	
	/**
	 * Dona l'usuari qui pertany la sessió. 
	 * @return Entitat Usuari de qui ha fet <i>loggin</i>, <b>no</b> gestionada
	 *  pel gestor de persistència, o <code>null</code> si l'usuari no s'ha
	 *  identificat
	 */	
	public Usuari getUsuari() {
		
		return this.usuari;
	}
	
	/**
	 * Dona el nom d'usuari d'aquell a qui pertany la sessió
	 * @return el nom amb el que s'ha identificat l'usuari,
	 *  o <code>null</code> si no ho ha fet
	 */
	public String getNomUsuari() {
		return this.usuari == null? null : this.usuari.getNomUsuari();
	}
	
}
