/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.controlador.usuaris;

import javax.annotation.security.RolesAllowed;
import javax.faces.bean.ManagedBean;

import edu.uoc.tcfj2ee_gatd.controlador.FiltreAbs;
import edu.uoc.tcfj2ee_gatd.model.peticions.entitats.Client;
import edu.uoc.tcfj2ee_gatd.model.usuaris.GATDExcepcioUsuaris;

/**
 * Controlador MVC, responsable de la navegació entre les pàgines d'administració
 * dels clients, per als usuaris Controlador.
 * @author Jordi Tejero
 *
 */
@ManagedBean(name = "tclients")
@RolesAllowed({"Controlador"})
public class ControladorTotsClients extends ControladorUsuaris<Client> {
	
	
	/**
	 * El filtre que dona el llistat dels clients.
	 */
	private FiltreClients filtreClients;	

	/**
	 * Constructor
	 * 
	 * @throws GATDExcepcioUsuaris
	 */
	public ControladorTotsClients() throws GATDExcepcioUsuaris {
		super(Client.class);
		this.filtreClients = new FiltreClients();
		
	}

	/* Publica el filtre
	 * @see edu.uoc.tcfj2ee_gatd.controlador.usuaris.ControladorUsuaris#getFiltre()
	 */
	@Override
	public FiltreAbs getFiltre() {
		return this.filtreClients;
	}

	/**
	 * Navegació web. Passa a la seva pàgina d'administració del client actual
	 * @return la pàgina d'administració de client
	 */
	public String administra() {
		
		return "administraClient.xhtml";
	}
	
	/**
	 * Navegació web. Alta del client actual i continuar en la mateixa
	 * pàgina.
	 * <p>
	 * Si l'alta té èxit, s'ofereix la fitxa del client buida. Si no en té,
	 * les dades entrades romanen. 
	 * </p>
	 * <p>
	 * Si ja hi havia un client amb el mateix nom, mostra un missatge d'error.
	 * </p>
	 * @return <code>null</code>
	 */
	public String alta() {

		if(super.fesAlta())
			this.setActual(new Client());
				
		return null;
	}	

}
