/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.controlador.usuaris;

import java.io.Serializable;

import edu.uoc.tcfj2ee_gatd.controlador.FiltreAbs;

/**
 * Filtre pel llistat de tècnics
 * @author Jordi Tejero
 *
 */
public class FiltreTecnics extends FiltreAbs implements Serializable {
	
	
	/**
	 * Versió de serialització
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Nom d'usuari del tècnic a cercar. Pot ser <code>null</code>, que 
	 * indica no restricció per aquest motiu.
	 */
	private String nomUsuari;
	
	/**
	 * Patró LIKE dels noms complets dels tècnics a cercar. Pot ser 
	 * <code>null</code>, que* indica no restricció per aquest motiu.
	 */
	private String patroNomComplet;
	
	/**
	 * Codi de la seu a la que any de pertànyer els tècnics a cercar.
	 * Pot ser <code>null</code>, que indica no restricció per aquest motiu.
	 */
	private String codiSeu;
	
	/**
	 * Si <code>true</code>, es cercaran sols els tècnics en situació
	 * d'activitat.
	 */
	private boolean solsActius;

	/**
	 * Constructor. El filtre creat no imposa cap restricció.
	 */
	public FiltreTecnics() {
		super();
	}

	/* Elimina tots els atributs corresponents a criteris de filtrat. La
	 * sentència JPQL trobarà tots els tècnics.
	 * @see edu.uoc.tcfj2ee_gatd.controlador.FiltreAbs#ini()
	 */
	@Override
	protected void ini() {
		this.nomUsuari = null;
		this.patroNomComplet = null;
		this.codiSeu = null;
		this.solsActius = false;
	}
	
	/**
	 * Accessor d'escriptura a l'atribut nomUsuari
	 * @param nomUsuari el nou valor de l'atribut nomUsuari
	 */
	public void setNomUsuari(String nomUsuari) {
		this.nomUsuari = nomUsuari;
	}

	/**
	 * Accessor de lectura de l'atribut nomUsuari
	 * @return el valor de l'atribut <code>nomUsuari</code>
	 */
	public String getNomUsuari() {
		return this.nomUsuari;
	}

	/**
	 * Accessor d'escriptura a l'atribut patroNomComplet
	 * @param patroNomComplet el nou valor de l'atribut patroNomComplet
	 */
	public void setPatroNomComplet(String patroNomComplet) {
		this.patroNomComplet = patroNomComplet;
	}

	/**
	 * Accessor de lectura de l'atribut patroNomComplet
	 * @return el valor de l'atribut <code>patroNomComplet</code>
	 */
	public String getPatroNomComplet() {
		return this.patroNomComplet;
	}

	/**
	 * Accessor d'escriptura a l'atribut codiSeu
	 * @param codiSeu el nou valor de l'atribut codiSeu
	 */
	public void setCodiSeu(String codiSeu) {
		this.codiSeu = codiSeu;
	}

	/**
	 * Accessor de lectura de l'atribut codiSeu
	 * @return el valor de l'atribut <code>codiSeu</code>
	 */
	public String getCodiSeu() {
		return this.codiSeu;
	}

	/**
	 * Indica si el filtre mostra sols els tècnics actius
	 * @param solsActius el nou valor de l'atribut solsActius
	 */
	public void setSolsActius(boolean solsActius) {
		this.solsActius = solsActius;
	}
	
	/**
	 * Accessor de lectura de l'atribut solsActius
	 * @return el valor de l'atribut <code>solsActius</code>
	 */
	public boolean isSolsActius() {
		return this.solsActius;
	}
	
	/**
	 * Munta la sentència JPQL de selecció, en funció dels atributs
	 * del filtre
	 * @return cadena amb la sentència JPQL
	 */
	@Override
	public String getJPQL() {
		this.netejaJPQL();
		this.posaSelect("SELECT t FROM Tecnic t");
		
		// Mai no mostrem els tècnics de baixa
		this.posaAnd("t.dataBaixa IS NULL");		
		this.posaAnd("t.nomUsuari", this.nomUsuari);		
		this.posaAndLike("t.contacte.nom", this.patroNomComplet);		
		this.posaAnd("t.seu.codi", this.codiSeu);		
		if(this.solsActius)
			this.posaAnd("t.actiu = true");		
		
		/*this.getLogger().finer("FiltreTecnics#getJPQL() ->" + Util.LS + 
				this.getSentencia());*/		
		
		return this.getSentencia();
	}
	
}
