/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.model.atencio;

import java.util.LinkedList;
import java.util.List;

import edu.uoc.tcfj2ee_gatd.model.comu.Util;

/**
 * Un missatge de correu senzill. Consta d'un text, un remitent, un
 * assumpte i una llista de destinataris
 * @author Jordi Tejero
 *
 */
 public class EMailSenzill  {
	
	/**
	 * Text del missatge
	 */
	private final StringBuilder text;

	/**
	 * Llista de destinataris
	 */
	private final List<String> to;
	
	
	/**
	 * Remitent. <b>No</b> pot ser <code>null</code>
	 */
	private String from;
	
	/**
	 * Assumpte. <b>No</b> pot ser <code>null</code>
	 */
	private String subject;
	
	/**
	 * Constructor
	 */
	public EMailSenzill() {
		super();
		this.text = new StringBuilder();
		this.subject = "";
		this.to = new LinkedList<String>();	
	}

	/**
	 * Accessor de lectura a from
	 * @return el valor de from
	 */
	public String getFrom() {
		return this.from;
	}

	/**
	 * Accessor de escriptura a from
	 * @param from String el nuevo valord de from
	 */
	public void setFrom(String from) {
		this.from = from;
	}

	/**
	 * Accessor de lectura a text
	 * @return el valor de text
	 */
	public StringBuilder getText() {
		return this.text;
	}

	/**
	 * Accessor de lectura a la llista de destinataris "to"
	 * @return el valor de to
	 */
	public List<String> getTo() {
		return this.to;
	}
	
	/**
	 * Dona la llista d'adreces dels destinataris, separades per comes
	 * @return llista de cadenes amb les adreces
	 */
	public String listTo() {
		return this.list(this.to);
	}

	
	/**
	 * Accesor de lectura a subject
	 * @return el valor de subject
	 */
	public String getSubject() {
		return this.subject;
	}

	/**
	 * Accessor d'scritura a subject
	 * @param subject String el nuevo valord de subject
	 */
	public void setSubject(String about) {
		this.subject = about;
	}

	/**
	 * Crea el llistat separat per comes, d'una seqüència d'objectes
	 * @param <T> tipus dels objectes de la seqüència
	 * @param sequencia la seqüència de N objectes O1 ... Oi ... ON 
	 * @return cadena [S1], ..., [Si], ...., [SN], on [Si] es la 
	 *  representació textual de Oi, i = 1...N
	 */
	private <T> String list(Iterable<T> sequencia) {
		StringBuilder result = new StringBuilder();
		for(T o : sequencia) {
			if(result.length() != 0)
				result.append(", ");
			result.append(o);
		}	
		
		return result.toString();
	}
	

	/*  Representació textual de l'objecte, per a depuració i proves.
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return 
		"[" + this.getClass().getSimpleName() + "]" + Util.LS +
		"From: " + this.from + Util.LS + 
		"To: " + this.listTo() + Util.LS + 
		"Subject: " + this.subject + Util.LS +
		"Text: " + this.text;
	}

}
