package edu.uoc.tcfj2ee_gatd.model.atencio;

import java.util.Properties;
import java.util.logging.Level;

import javax.annotation.PostConstruct;
import javax.ejb.Stateless;
import javax.mail.Authenticator;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import edu.uoc.tcfj2ee_gatd.configuracio.Configuracio;
import edu.uoc.tcfj2ee_gatd.configuracio.GestorConfiguracio;
import edu.uoc.tcfj2ee_gatd.model.comu.EJBGestor;



/**
 * Enviador de missatges de correu SMTP senzills.
 * 
 * @author Jordi Tejero
 * 
 */
@Stateless
public class GestorEMail extends EJBGestor {


    /**
     * Propietats JavaMail
     */
    private Properties mailPrp;
    
    // Autentificador per la sessió SMTP
	private Authenticator auth; 
    
	/**
     * Constructor
     * 
     * @throws GATDExcepcioAtencio en cas d'error
     */
    public GestorEMail() throws GATDExcepcioAtencio {
    	super();
    	
    }
        
    /**
     * Incialització a partir de la instància única <code>Configuracio</code>
     */
    @PostConstruct
    public void init() {
    	// Configuració del sistema, "singleton"
    	Configuracio cfg = GestorConfiguracio.getConfig();
    	assert cfg != null;
    	/*
    	 * Si la configuració defineix usuari i clau d'accés al servidor
    	 * SMTP, creem un autentificador
    	 */
	    if(cfg.getMailUsuari() != null && cfg.getMailUsuari() != null) {
	            this.auth = new AtentificadorEMailSenzill(
	            		cfg.getMailUsuari(), cfg.getMailClau()
	           );
	    }
	    else
	    	this.auth = null;
	    
	    /*
	     * Creem un conjunt de propietats per a la sessió SMTP de JavaMail 1.4.
	     * Referència: JavaMail-1.4.pdf, appendix A.
	     */
	    this.mailPrp = new Properties();
	    this.mailPrp.setProperty("mail.debug", "false");
	    this.mailPrp.setProperty("mail.transport.protocol", "smtp");
	    this.mailPrp.setProperty("mail.smtp.host", cfg.getMailHost());
	    this.mailPrp.setProperty("mail.smtp.auth", 
	    		Boolean.toString(this.auth != null));	    
    	
    }
    
    /**
     * Envia un missatge amb JavaMail
     * @param email el missatge
     * @throws GATDExcepcioAtencio cas d'error en l'enviament SMTP
     */
    public void envia(EMailSenzill email) throws GATDExcepcioAtencio 
    {        
    	this.getLogger().finer(this.getClass().getSimpleName() 
    			+ "#envia: " + email);
    	
    	// Sessió JavaMail
    	Session mailSession;
    	// Transport JavaMail  
    	Transport transport = null;
    	// Missatge JavaMail
    	MimeMessage message;       	  
    	
    	if(email.getTo().isEmpty())
    		throw new GATDExcepcioAtencio("aten_excp_MailNoDestins");
    	
    	mailSession = this.auth == null?
    		Session.getInstance(this.mailPrp)
    		: Session.getInstance(this.mailPrp, this.auth);
        
	    try {   
	        transport = mailSession.getTransport();
	        message = new MimeMessage(mailSession);    	        
      
    		for(String to : email.getTo())
				message.addRecipient(Message.RecipientType.TO,
				        new InternetAddress(to));    	
			message.setFrom(new InternetAddress(email.getFrom()));
			message.setSubject(email.getSubject());
			message.setContent(email.getText().toString(), "text/plain");
	
	        transport.connect();
	        transport.sendMessage(message, message.getAllRecipients());
	        transport.close();
    	}
    	catch(MessagingException e) {
    		this.getLogger().log(Level.SEVERE, "aten_excp_EnviaMail", e);
    		throw new GATDExcepcioAtencio(e, "aten_excp_EnviaMail", e.toString());
    	}    	
    	finally {
    		if(transport != null && transport.isConnected())
				try {
					transport.close();
				} catch (MessagingException e) {
					this.getLogger().log(Level.SEVERE, "transport.close()", e);
		    		throw new GATDExcepcioAtencio(e, "aten_excp_EnviaMail", 
		    			e.toString());
				}
    	}    	
    
    }
	
}

