/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.model.comu;

import java.util.List;
import java.util.logging.Logger;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;



/**
 * Session Bean que implementa lògica de negoci
 * @author Jordi Tejero
 *
 */
public class EJBGestor {
	
	/**
	 * Gestor de la persistència
	 */
	@PersistenceContext(unitName = "GATD_PU")
	private EntityManager em;
	
	/**
	 * Logger 
	 */
	private final Logger logger;
	

	/**
	 * Constructor. Crea el logger.
	 */
	public EJBGestor() {
		this.logger = Logger.getLogger(this.getClass().getCanonicalName());
		assert this.logger != null;
	}
	
	/**
	 * Dona d'alta una nova entitat. Si no tenia la data d'alta establerta, li 
	 * posa la data del moment.
	 * @param <E> el tipus de la entitat
	 * @param entitat la entitat
	 * @return la mateixa entitat <code>entitat</code>
	 */
	public <E extends Entitat> E alta(E entitat) {
		if(!entitat.isAlta())
			entitat.alta();
		this.em.persist(entitat);
		this.em.flush(); 
		
		//this.getLogger().finest("EJBGestor#alta " + entitat);
		return entitat;
	}

	/**
	 * Publica el gestor de persistència per a ús de les
	 * classes derivades
	 * @return el gestor de persistència
	 */
	protected EntityManager getEM() {
		return this.em;
	}
	
	/**
	 * Dona una llista d'entitats a partir d'una sentència JPQL de
	 * selecció
	 * @param <E> el tipus de les entitats
	 * @param jpql la sentència JPQL que dona les entitats
	 * @return una llista, pot ser que buida, amb les entitats recuperades
	 *  del gestor de persistència per la sentència JPQL.
	 */
	@SuppressWarnings("unchecked")
	public <E extends Entitat> List<E> llista(String jpql) {
		return this.em.createQuery(jpql).getResultList();		
	}
	
	/**
	 * Dona una llista d'entitats a partir del nom d'una <i>query</i>
	 * @param <E> el tipus de les entitats
	 * @param nomQuery el nom de la <i>query</i>
	 * @return una llista, pot ser que buida, amb les entitats recuperades
	 *  del gestor de persistència per la <i>query</i>.
	 */
	@SuppressWarnings("unchecked")
	public <E extends Entitat> List<E> llistaNominal(String nomQuery) {
		return this.em.createNamedQuery(nomQuery).getResultList();
	}

	/**
	 * Accessor de lectura de l'atribut logger
	 * @return el valor de l'atribut <code>logger</code>
	 */
	public Logger getLogger() {
		return this.logger;
	}
}
