/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.model.comu;

/**
 * Excepció específica de l'aplicació.
 * <p>
 * Implementa la internacionalització segons la recomanació I18N.
 * Els constructors, en lloc de prendre un missatge, reben una
 * clau par a trobar-lo a "resource bundle" pertinent.
 * </p>
 * <p>
 * A més a més, el missatge pot ser una cadena per a formatar un seguit
 * d'arguments addicionals, segons la funció <code>String.format(...)</code>
 * 
 * </p>
 * @author Jordi Tejero
 *
 */
public class GATDExcepcio extends Exception implements IMissatgeAmbFormat {

	/**
	 * Requerit al implementar <code>Serializable</code>
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Clau del missatge al "resource bundle" de l'aplicació. Pot ser
	 * <code>null</code> si l'exepció no té cap missatge.
	 */
	private final String clau;
	
	/**
	 * Array amb els arguments addicionals del missatge. Pot ser
	 * <code>null</code> si l'exepció no té cap missatge.
	 */
	private final Object[] aargs;
	


	/**
	 * Constructor amb causa, clau de missatge i arguments addicionals.
	 * <p>
	 * Si no s'indica la clau del missatge i la causa C es una altra 
	 * <code>GATDExcepcio</code>, la nova excepció agafa la clau i els 
	 * arguments addicionals de C. 
	 * </p>
	 * <p>
	 * Si no s'indica la clau del missatge i la causa C no es una altra 
	 * <code>GATDExcepcio</code>, el missatge de la nova excepció será
	 * <code>C.toString()</code>
	 * </p>
	 * @param cause la causa C de l'excepció
	 * @param clau clau del missatge, que pot ser <code>null</code>
	 * @param aargs arguments addicionals del missatge, que poden ser 
	 *  <code>null</code>
	 */
	public GATDExcepcio(Throwable cause, String clau,  Object... aargs) {				
		super(clau, cause);
		
		Object[] nousAargs = aargs;
		
		if(clau == null) {
			if(cause instanceof GATDExcepcio) {
				GATDExcepcio gcause = (GATDExcepcio)cause;
				this.clau = gcause.getClau();
				nousAargs = gcause.getAargs();
			}
			else
				this.clau = cause.toString();
		}
		else
			this.clau = clau;
		this.aargs = nousAargs;
	}

	/**
	 * Constructor amb la clau del missatge i arguments addicionals
	 * @param clau clau del missatge, que pot ser <code>null</code>
	 * @param aargs arguments addicionals del missatge, que poden ser 
	 *  <code>null</code>
	 * 
	 */
	public GATDExcepcio(String clau, Object...aargs) {
		this(null, clau, aargs);
	}

	/**
	 * Constructor a partir d'una altra GATDExcepcio G, sense clau ni arguments.
	 * Agafa els de G.
	 * @param gcause causa de l'excepció
	 * 
	 */
	public GATDExcepcio(GATDExcepcio gcause) {
		this(gcause, gcause.getClau(), gcause.getAargs());
	}


	
	/* (non-Javadoc)
	 * @see edu.uoc.tcfj2ee_gatd.model.comu.IMissatgeAmbFormat#getClau()
	 */
	public String getClau() {
		return this.clau;
	}

	/* (non-Javadoc)
	 * @see edu.uoc.tcfj2ee_gatd.model.comu.IMissatgeAmbFormat#getAargs()
	 */
	public Object[] getAargs() {
		return this.aargs;
	}

}
