/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.model.peticions.entitats;

import java.util.Date;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;

import edu.uoc.tcfj2ee_gatd.model.comu.Entitat;
import edu.uoc.tcfj2ee_gatd.model.comu.Util;
import edu.uoc.tcfj2ee_gatd.model.usuaris.entitats.Usuari;

/**
 * Queixa sobre una petició, feta per un client o un controlador
 * @author Jordi Tejero
 *
 */
@Entity
public class Queixa extends Entitat {
	
	/**
	 * Petició a la que pertany la queixa
	 */
	@Id
	private Peticio peticio;

	/**
	 * Text descriptiu de la queixa
	 */
	@Column(length = 300)
	private String descripcio;
	
	/**
	 * Usuari que va fer la queixa
	 */
	private Usuari autor;
	
	/**
	 * Constructor sense paràmetres, obligatori
	 */
	public Queixa() {
		super();
	}
	
	/**
	 * Constructor amb tots els atributs. La queixa queda enllaçada
	 * bidireccionalment amb la petició
	 * @param peticio
	 * @param descripcio
	 * @param autor
	 * @param dataAlta
	 * @param dataBaixa
	 */
	public Queixa(Peticio peticio, String descripcio, 
			Usuari autor, Date dataAlta, Date dataBaixa) {
		this();
		
		if(dataAlta != null)
			this.setDataAlta(dataAlta);
		if(dataBaixa != null)
			this.setDataBaixa(dataBaixa);
		this.lliga(peticio);
		this.descripcio = descripcio;
		this.autor = autor;
	}
	
	/**
	 * Constructor amb la petició i l'autor de la queixa 
	 * @param peticio la petició a la que es refereix la queixa
	 * @param autor l'usuari autor de la queixa
	 */
	public Queixa(Peticio peticio, Usuari autor) {
		this(peticio, null, autor, null, null);
	}

	/**
	 * Associa la queixa amb una petició, bidireccinalment
	 * @param p la petició
	 */
	private void lliga(Peticio p) {
		this.peticio = p;
		p.setQueixa(this);
	}
	
	/**
	 * Accessor de lectura de l'atribut peticio
	 * @return el valor de l'atribut <code>peticio</code>
	 */
	public Peticio getPeticio() {
		return this.peticio;
	}

	/**
	 * Estableix la petició a la que pertany la queixa.
	 * Si estava ja d'alta, era una queixa preexistent i ho comuniquem
	 * al possible controlador observador 
	 * @param peticio el nou valor de l'atribut peticio
	 */
	public void setPeticio(Peticio peticio) {
		this.peticio = peticio;
	}

	/**
	 * Accessor de lectura de l'atribut descripcio
	 * @return el valor de l'atribut <code>descripcio</code>
	 */
	public String getDescripcio() {
		return this.descripcio;
	}

	/**
	 * Accessor d'escriptura a l'atribut descripcio
	 * @param descripcio el nou valor de l'atribut descripcio
	 */
	public void setDescripcio(String descripcio) {
		this.descripcio = descripcio;
		assert this.getLogger() != null;
		this.getLogger().finer("Queixa#setDescripcio \"" + this.descripcio + "\"");
	}

	/**
	 * Accessor de lectura de l'atribut autor
	 * @return el valor de l'atribut <code>autor</code>
	 */
	public Usuari getAutor() {
		return this.autor;
	}

	/**
	 * Accessor d'escriptura a l'atribut autor
	 * @param autor el nou valor de l'atribut autor
	 */
	public void setAutor(Usuari autor) {
		this.autor = autor;
	}
	
	/* Representació textual de l'objecte, per a depuració i proves.
	 * @see edu.uoc.tcfj2ee_gatd.model.comu.Entitat#toString()
	 */
	@Override
	public String toString() {
		return 
		super.toString() + Util.LS +
		"Petició: " + 
		(this.peticio == null? null : this.peticio.getReferencia()) + Util.LS +
		"Autor: " + 
		(this.autor == null? null : this.autor.getNomUsuari()) + Util.LS +
		"Descripció: " + this.descripcio
		;
	}

}
