/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.model.peticions.entitats;

import java.util.ArrayList;
import java.util.Collection;

import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;

import edu.uoc.tcfj2ee_gatd.model.atencio.entitats.Tecnic;
import edu.uoc.tcfj2ee_gatd.model.comu.Entitat;
import edu.uoc.tcfj2ee_gatd.model.comu.Util;
import edu.uoc.tcfj2ee_gatd.model.usuaris.entitats.Adreça;



/**
 * Una seu o taller dels tècnics
 * @author Jordi Tejero
 *
 */
@Entity
@NamedQueries({
	@NamedQuery(name="seusAlta", 
	query="SELECT s FROM Seu s WHERE s.dataBaixa IS NULL ORDER BY s.nom")
})
public class Seu extends Entitat {

	/**
	 * El codi de la seu
	 */
	@Id
	@Column(length = 10)
	private String codi;
	
	/**
	 * El nom de la seu
	 */
	@Column(length = 20)
	private String nom;
	
	/**
	 * Els tècnics de la seu
	 */
	@OneToMany(mappedBy = "seu")
	private Collection<Tecnic> tecnics;
	
	/**
	 * L'adreça de la seu
	 */
	@OneToOne(cascade = CascadeType.ALL)
	private Adreça adreça;
	
	/**
	 * Constructor sense paràmetres, obligatori. La seu creada no té
	 * codi, tècnics ni cap altra mena de dada.
	 */	
	public Seu() {
		super();
		this.tecnics = new ArrayList<Tecnic>();
	}

	/**
	 * Constructor amb el codi de la seu, nom i l'adreça
	 * @param codi el codi de la seu
	 * @param nom el nom de la seu
	 * @param adreça l'adreça de la seu
	 */
	public Seu(String codi, String nom, Adreça adreça) {
		this();
		this.codi = codi;
		this.nom = nom;
		this.adreça = adreça;
	}

	/**
	 * Accessor de lectura de l'atribut codi
	 * @return el valor de l'atribut <code>codi</code>
	 */
	public String getCodi() {
		return this.codi;
	}

	/**
	 * Accessor d'escriptura a l'atribut codi
	 * @param codi el nou valor de l'atribut codi
	 */
	public void setCodi(String codi) {
		/*this.getLogger().finer(this.getClass().getSimpleName() 
			+ "#setCodi(" + codi + ")");*/
		this.codi = codi;
	
	}

	/**
	 * Accessor de lectura de l'atribut nom
	 * @return el valor de l'atribut <code>nom</code>
	 */
	public String getNom() {
		return this.nom;
	}


	/**
	 * Accessor de lectura de l'adreça
	 * @return el valor de l'atribut <code>adreça</code>
	 */
	public Adreça getAdreça() {
		return this.adreça;
	}

	/**
	 * Publica els tècnics
	 * @return la col·lecció dels tècnics
	 */
	public Collection<Tecnic> getTecnics() {
		return this.tecnics;
	}
	
	/**
	 * Ajuda al "logging". Dona una representació abreujada de <code>this</code>
	 * @return cadena amb el nom de la classe i el codi de la seu
	 */
	public String getBreu() {
		return "[" + this.getClass().getSimpleName() + "] " + this.codi;
	}

	/* Dos objectes representen la mateixa seu si tenen el mateix codi
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object altre) {
		if(altre == null)
			return false;
		if(!(altre instanceof Seu))
			return false;
		return
			this.codi != null && this.codi.equals(((Seu)altre).getCodi());
	}

	/* Codi de dispersió compatible amb equals(...)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		return this.codi == null? 0 : this.codi.hashCode();
	}
	
	/*  Representació textual de l'objecte, per a depuració i proves.
	 * @see edu.uoc.tcfj2ee_gatd.model.comu.Entitat#toString()
	 */
	@Override
	public String toString() {
		return 
		super.toString() + Util.LS +
		"Codi: " + this.codi + ", nom: " + this.nom + ", " +
		"tècnics: " + this.tecnics.size() + Util.LS +
		"Adreça: " + this.adreça
		;
		
	}

	
}
