/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.model.usuaris.entitats;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToOne;

import edu.uoc.tcfj2ee_gatd.model.comu.Util;



/**
 * Una adreça. Les considerem totes diferents, sense mirar si representen
 * o no el mateix domicili.
 * 
 * @author Jordi Tejero
 *
 */
@Entity
public class Adreça {

	/**
	 * Identificador únic de l'adreça, donat pel sistema
	 */
	@Id	@GeneratedValue
	private int id;
	
	/**
	 * El districte postal 
	 */
	@ManyToOne
	private DistrictePostal dp;

	/**
	 * El nom de la població
	 */
	@Column(length = 40)
	private String poblacio;
	
	/**
	 * El nom del carrer
	 */
	@Column(length = 40)
	private String carrer;

	/**
	 * La numeració del carrer. Pot ser quelcom similar a "22b" o "25-27"
	 */
	@Column(length = 20)
	private String numero;

	/**
	 * El pis. Per exemple, "3", "3er." o "àtic"
	 */
	@Column(length = 10)
	private String pis;

	/**
	 * La numeració de la porta, com ara "3" o "D".
	 */
	@Column(length = 10)
	private String porta;

	/**
	 * Constructor sense paràmetres, obligatori
	 * <p>Crea l'atribut districte postal, sense codi.
	 * </p>
	 */
	public Adreça() {
		super();
		this.dp = new DistrictePostal();
	}

	
	/**
	 * Constructor
	 * @param poblacio
	 * @param dp
	 * @param carrer
	 * @param numero
	 * @param pis
	 * @param porta
	 */
	public Adreça(String poblacio, DistrictePostal dp, String carrer,
			String numero, String pis, String porta) {
		this();
		this.poblacio = poblacio;
		this.dp = dp;
		this.carrer = carrer;
		this.numero = numero;
		this.pis = pis;
		this.porta = porta;
	}

	
	
	/**
	 * Crea una copia d'una altra adreça, tret de l'identificador
	 * @param altra l'altra adreça
	 */
	public Adreça(Adreça altra) {
		this(altra.poblacio, altra.dp, altra.carrer, altra.numero,
			altra.pis, altra.porta)
		;
	}


	/**
	 * Accessor de lectura de l'atribut id
	 * @return el valor de l'atribut <code>id</code>
	 */
	public int getId() {
		return this.id;
	}


	/**
	 * Accessor de lectura de l'atribut poblacio
	 * @return el valor de l'atribut <code>poblacio</code>
	 */
	public String getPoblacio() {
		return this.poblacio;
	}


	/**
	 * Accessor d'escriptura a l'atribut poblacio
	 * @param poblacio el nou valor de l'atribut poblacio
	 */
	public void setPoblacio(String poblacio) {
		this.poblacio = poblacio;
	}


	/**
	 * Accessor de lectura de l'atribut dp
	 * @return el valor de l'atribut <code>dp</code>
	 */
	public DistrictePostal getDp() {
		return this.dp;
	}


	/**
	 * Accessor d'escriptura a l'atribut dp
	 * @param dp el nou valor de l'atribut dp
	 */
	public void setDp(DistrictePostal dp) {
		this.dp = dp;
	}


	/**
	 * Accessor de lectura de l'atribut carrer
	 * @return el valor de l'atribut <code>carrer</code>
	 */
	public String getCarrer() {
		return this.carrer;
	}


	/**
	 * Accessor d'escriptura a l'atribut carrer
	 * @param carrer el nou valor de l'atribut carrer
	 */
	public void setCarrer(String carrer) {
		this.carrer = carrer;
	}


	/**
	 * Accessor de lectura de l'atribut numero
	 * @return el valor de l'atribut <code>numero</code>
	 */
	public String getNumero() {
		return this.numero;
	}


	/**
	 * Accessor d'escriptura a l'atribut numero
	 * @param numero el nou valor de l'atribut numero
	 */
	public void setNumero(String numero) {
		this.numero = numero;
	}


	/**
	 * Accessor de lectura de l'atribut pis
	 * @return el valor de l'atribut <code>pis</code>
	 */
	public String getPis() {
		return this.pis;
	}


	/**
	 * Accessor d'escriptura a l'atribut pis
	 * @param pis el nou valor de l'atribut pis
	 */
	public void setPis(String pis) {
		this.pis = pis;
	}


	/**
	 * Accessor de lectura de l'atribut porta
	 * @return el valor de l'atribut <code>porta</code>
	 */
	public String getPorta() {
		return this.porta;
	}


	/**
	 * Accessor d'escriptura a l'atribut porta
	 * @param porta el nou valor de l'atribut porta
	 */
	public void setPorta(String porta) {
		this.porta = porta;
	}
	
	/**
	 * Dona el carrer, pis, número i porta en una forma compta,
	 * adient pels llistats
	 * @return cadena en format <i>carrer numero [, pis] [ porta]</i> 
	 */
	public String getDadesCarrer() {
		StringBuilder result = new StringBuilder();
		
		result.append(this.carrer).append(" ")
		.append(this.numero);
		if(this.pis != null && !this.pis.isEmpty())
			result.append(", ").append(this.pis);
		
		if(this.porta != null && this.porta.isEmpty())
			result.append(" ").append(this.porta);
				
		return result.toString();
		
	}
	
	/**
	 * Representació textual de l'adreça. No forma part de la lógica 
	 * de negoci.
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return 
		"[" + this.getClass().getSimpleName() + "]" + Util.LS +
		"Carrer " + this.carrer + " número " + this.numero + ", " +
		"pis " + this.pis + " porta " + this.porta + Util.LS +
		this.dp.getCodi() + " " + this.poblacio
		;
			
	}

}
