/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.model.usuaris.entitats;

import javax.persistence.Column;
import javax.persistence.Embeddable;

import edu.uoc.tcfj2ee_gatd.model.comu.Util;

/**
 * Dades de contacte amb una persona o entitat. Inclou el seu nom complet.
 * @author Jordi Tejero
 *
 */
@Embeddable
public class Contacte {
	/**
	 * El nom complet 
	 */
	@Column(length = 80)
	private String nom;
	/**
	 * El número de telèfon fix
	 */
	@Column(length = 20)
	private String telefon;
	/**
	 * El número de telèfon mòbil
	 */
	@Column(length = 20)
	private String mobil;
	/**
	 * L'adreça de correu electrònic
	 */
	@Column(length = 40)
	private String eMail;
	
	/**
	 * Constructor sense paràmetres, obligatori
	 */
	public Contacte() {
		// no res
	}
	
	/**
	 * Constructor
	 * @param nom
	 * @param telefon
	 * @param mobil
	 * @param eMail
	 */
	public Contacte(String nom, String telefon, String mobil, String eMail) {
		super();
		this.nom = nom;
		this.telefon = telefon;
		this.mobil = mobil;
		this.eMail = eMail;
	}
	
	/**
	 * Crea una copia d'un altre contacte
	 * 
	 * @param altre l'altre contacte
	 */
	public Contacte(Contacte altre) {
		this(altre.nom, altre.telefon, altre.mobil, altre.eMail);
	}
	
	/**
	 * Accessor de lectura del nom del contacte
	 * @return el valor de <code>nom</nom>
	 */
	public String getNom() {
		return this.nom;
	}
	/**
	 * Accessor d'escriptura al nom del contacte
	 * @param nom el nou valor de <code>nom</nom>
	 */
	public void setNom(String nom) {
		this.nom = nom;
	}
	/**
	 * Accessor de lectura del telèfon fix
	 * @return el valor de <code>telefon</code>
	 */
	public String getTelefon() {
		return this.telefon;
	}
	/**
	 * Accessor d'escriptura del telèfon fix
	 * @param telefon el nou valor de <code>telefon</code>
	 */
	public void setTelefon(String telefon) {
		this.telefon = telefon;
	}
	/**
	 * Accessor de lectura de mobil
	 * @return el valor de mobil
	 */
	public String getMobil() {
		return this.mobil;
	}
	/**
	 * Accessor d'escriptura del telèfon mòbil
	 * @param mobil el nou valor de <code>mobil</code>
	 */
	public void setMobil(String mobil) {
		this.mobil = mobil;
	}
	/**
	 * Accessor de lectura de l'adreça e-mail
	 * @return el valor de l'atribut <code>eMail</code>
	 */
	public String geteMail() {
		return this.eMail;
	}
	/**
	 * Accessor d'escriptura a l'adreça e-mail
	 * @param eMail el nou valor de l'atribut <code>eMail</code>
	 */
	public void seteMail(String eMail) {
		this.eMail = eMail;
	}
	
	/* 
	 * Representació textual de l'objecte, a efectes de depuració i proves. 
	 * No forma part de la lògica del negoci
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return 
		"[" + this.getClass().getSimpleName() + "]. " 
	    + "Nom: " + this.nom + Util.LS 
	    + "Telèfon: " + this.telefon  
	    + ", mòbil: " + this.mobil 
	    + ", e-mail: " + this.eMail
	    ;
		
	}

	
}
