/**
 * 
 */
package edu.uoc.tcfj2ee_gatd.model.usuaris.entitats;

import javax.persistence.Column;
import javax.persistence.Embeddable;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;

/**
 * Document d'identitat d'un usuari
 * @author Jordi Tejero
 *
 */


@Embeddable
public class DocIdentitat {
	/**
	 * Enumeració dels tipus de document d'identitat
	 * @author Jordi Tejero
	 *
	 */
	public enum TIPUS {
		DNI, PASSAPORT, LF;	
	}
	
	/**
	 * Tipus del document d'identitat.
	 * A la taula de persistència, serà una cadena
	 */
	@Enumerated(EnumType.STRING)
	@Column(length = 10)
	private TIPUS tipus;
	
	/**
	 * Número del document d'identitat
	 */
	@Column(length = 17)
	private String numero;
	
	/**
	 * Constructor sense paràmetres, obligatori
	 */
	public DocIdentitat() {
		// no res
	}
	
	/**
	 * Constructor a partir de tipus i número del document
	 * @param tipus el nom del tipus
	 * @param numero el número
	 */
	public DocIdentitat(String tipus, String numero) {
		this.tipus = TIPUS.valueOf(tipus);
		this.numero = numero;
	}

	/**
	 * Accessor de lectura
	 * @return el tipus del document
	 */
	public TIPUS getTipus() {
		return this.tipus;
	}

	/**
	 * Accessor d'escriptura del tipus
	 * @param tipus el nou tipus
	 */
	public void setTipus(TIPUS tipus) {
		this.tipus = tipus;
	}
	
	/**
	 * Estableix el tipus a partir del seu nom
	 * @param nomTipus el nom del tipus
	 * @throws IllegalArgumentException si <code>nomTipus</code> no és el
	 *  nom de cap element de l'enumarció <code>TIPUS</code>
	 * @throws NullPointerException si <code>nomTipus</code> es
	 *  <code>null</code>. 
	 */
	public void setTipus(String nomTipus) {
		this.setTipus(TIPUS.valueOf(nomTipus));
	}

	/**
	 * Accessor de lectura del número
	 * @return el número del document
	 */
	public String getNumero() {
		return this.numero;
	}

	/**
	 * Accessor d'escriptura al número
	 * @param numero el nou número del document
	 */
	public void setNumero(String numero) {
		this.numero = numero;
	}
	
	/* Dos documents son el mateix si tenen igual tipus i número
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object altre) {
		if(altre == null || !(altre instanceof DocIdentitat))
			return false;
		if(this.tipus == null || this.numero == null)
			return false;
		DocIdentitat altreDI = (DocIdentitat)altre;
		
		return this.tipus.equals(altreDI.tipus) 
		&& this.numero.equals(altreDI.numero);
		
	}
	
	/* Codi de dispersió compatible amb equals(...)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		int x = this.tipus == null? 0 : this.tipus.hashCode();
		int y = this.numero == null? 0 : this.numero.hashCode();
		
		return x ^ y;
	}
	
	/** Representació textual de l'objecte, per a depuració i proves
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return
		"[" + this.getClass().getSimpleName() + "]. " +
		"Tipus " + this.tipus + ", número " + this.numero
		;
	}
}
